@@ -9,10 +9,11 @@ import static org.postgresql.util.internal.Nullness.castNonNull;
 
 import org.postgresql.jdbc.PgConnection;
 import org.postgresql.util.DriverInfo;
-import org.postgresql.util.ExpressionProperties;
 import org.postgresql.util.GT;
 import org.postgresql.util.HostSpec;
-import org.postgresql.util.LogWriterHandler;
+import org.postgresql.util.PGPropertyPasswordParser;
+import org.postgresql.util.PGPropertyServiceParser;
+import org.postgresql.util.PGPropertyUtil;
 import org.postgresql.util.PSQLException;
 import org.postgresql.util.PSQLState;
 import org.postgresql.util.SharedTimer;
@@ -36,11 +37,8 @@ import java.util.Enumeration;
 import java.util.Properties;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
-import java.util.logging.Formatter;
 import java.util.logging.Level;
 import java.util.logging.Logger;
-import java.util.logging.SimpleFormatter;
-import java.util.logging.StreamHandler;
 
 /**
  * <p>The Java SQL framework allows for multiple database drivers. Each driver should supply a class
@@ -65,7 +63,6 @@ public class Driver implements java.sql.Driver {
   private static final Logger PARENT_LOGGER = Logger.getLogger("org.postgresql");
   private static final Logger LOGGER = Logger.getLogger("org.postgresql.Driver");
   private static final SharedTimer SHARED_TIMER = new SharedTimer();
-  private static final String DEFAULT_PORT = "5432";
 
   static {
     try {
@@ -205,7 +202,7 @@ public class Driver implements java.sql.Driver {
    * @param info a list of arbitrary tag/value pairs as connection arguments
    * @return a connection to the URL or null if it isnt us
    * @exception SQLException if a database access error occurs or the url is
-   * {@code null}
+   *            {@code null}
    * @see java.sql.Driver#connect
    */
   @Override
@@ -246,8 +243,6 @@ public class Driver implements java.sql.Driver {
       return null;
     }
     try {
-      // Setup java.util.logging.Logger using connection properties.
-      setupLoggerFromProperties(props);
 
       LOGGER.log(Level.FINE, "Connecting with URL: {0}", url);
 
@@ -288,73 +283,13 @@ public class Driver implements java.sql.Driver {
     }
   }
 
-  // Used to check if the handler file is the same
-  private static /* @Nullable */ String loggerHandlerFile;
-
   /**
-   * <p>Setup java.util.logging.Logger using connection properties.</p>
-   *
-   * <p>See {@link PGProperty#LOGGER_FILE} and {@link PGProperty#LOGGER_FILE}</p>
-   *
+   *  this is an empty method left here for graalvm
+   *  we removed the ability to setup the logger from properties
+   *  due to a security issue
    * @param props Connection Properties
    */
   private void setupLoggerFromProperties(final Properties props) {
-    final String driverLogLevel = PGProperty.LOGGER_LEVEL.get(props);
-    if (driverLogLevel == null) {
-      return; // Don't mess with Logger if not set
-    }
-    if ("OFF".equalsIgnoreCase(driverLogLevel)) {
-      PARENT_LOGGER.setLevel(Level.OFF);
-      return; // Don't mess with Logger if set to OFF
-    } else if ("DEBUG".equalsIgnoreCase(driverLogLevel)) {
-      PARENT_LOGGER.setLevel(Level.FINE);
-    } else if ("TRACE".equalsIgnoreCase(driverLogLevel)) {
-      PARENT_LOGGER.setLevel(Level.FINEST);
-    }
-
-    ExpressionProperties exprProps = new ExpressionProperties(props, System.getProperties());
-    final String driverLogFile = PGProperty.LOGGER_FILE.get(exprProps);
-    if (driverLogFile != null && driverLogFile.equals(loggerHandlerFile)) {
-      return; // Same file output, do nothing.
-    }
-
-    for (java.util.logging.Handler handlers : PARENT_LOGGER.getHandlers()) {
-      // Remove previously set Handlers
-      handlers.close();
-      PARENT_LOGGER.removeHandler(handlers);
-      loggerHandlerFile = null;
-    }
-
-    java.util.logging.Handler handler = null;
-    if (driverLogFile != null) {
-      try {
-        handler = new java.util.logging.FileHandler(driverLogFile);
-        loggerHandlerFile = driverLogFile;
-      } catch (Exception ex) {
-        System.err.println("Cannot enable FileHandler, fallback to ConsoleHandler.");
-      }
-    }
-
-    Formatter formatter = new SimpleFormatter();
-
-    if ( handler == null ) {
-      if (DriverManager.getLogWriter() != null) {
-        handler = new LogWriterHandler(DriverManager.getLogWriter());
-      } else if ( DriverManager.getLogStream() != null) {
-        handler = new StreamHandler(DriverManager.getLogStream(), formatter);
-      } else {
-        handler = new StreamHandler(System.err, formatter);
-      }
-    } else {
-      handler.setFormatter(formatter);
-    }
-
-    Level loggerLevel = PARENT_LOGGER.getLevel();
-    if (loggerLevel != null) {
-      handler.setLevel(loggerLevel);
-    }
-    PARENT_LOGGER.setUseParentHandlers(false);
-    PARENT_LOGGER.addHandler(handler);
   }
 
   /**
@@ -462,7 +397,7 @@ public class Driver implements java.sql.Driver {
    * @throws SQLException if the connection could not be made
    */
   private static Connection makeConnection(String url, Properties props) throws SQLException {
-    return new PgConnection(hostSpecs(props), user(props), database(props), props, url);
+    return new PgConnection(hostSpecs(props), props, url);
   }
 
   /**
@@ -550,7 +485,15 @@ public class Driver implements java.sql.Driver {
    * @return Properties with elements added from the url
    */
   public static /* @Nullable */ Properties parseURL(String url, /* @Nullable */ Properties defaults) {
-    Properties urlProps = new Properties(defaults);
+    // priority 1 - URL values
+    Properties priority1Url = new Properties();
+    // priority 2 - Properties given as argument to DriverManager.getConnection()
+    // argument "defaults" EXCLUDING defaults
+    // priority 3 - Values retrieved by "service"
+    Properties priority3Service = new Properties();
+    // priority 4 - Properties loaded by Driver.loadDefaultProperties() (user, org/postgresql/driverconfig.properties)
+    // argument "defaults" INCLUDING defaults
+    // priority 5 - PGProperty defaults for PGHOST, PGPORT, PGDBNAME
 
     String urlServer = url;
     String urlArgs = "";
@@ -567,36 +510,45 @@ public class Driver implements java.sql.Driver {
     }
     urlServer = urlServer.substring("jdbc:postgresql:".length());
 
-    if (urlServer.startsWith("//")) {
+    if (urlServer.equals("//") || urlServer.equals("///")) {
+      urlServer = "";
+    } else if (urlServer.startsWith("//")) {
       urlServer = urlServer.substring(2);
+      long slashCount = urlServer.chars().filter(ch -> ch == '/').count();
+      if (slashCount > 1) {
+        LOGGER.log(Level.WARNING, "JDBC URL contains too many / characters: {0}", url);
+        return null;
+      }
       int slash = urlServer.indexOf('/');
       if (slash == -1) {
         LOGGER.log(Level.WARNING, "JDBC URL must contain a / at the end of the host or port: {0}", url);
         return null;
       }
-      urlProps.setProperty("PGDBNAME", URLCoder.decode(urlServer.substring(slash + 1)));
+      if (!urlServer.endsWith("/")) {
+        String value = urlDecode(urlServer.substring(slash + 1));
+        if (value == null) {
+          return null;
+        }
+        PGProperty.PG_DBNAME.set(priority1Url, value);
+      }
+      urlServer = urlServer.substring(0, slash);
 
-      String[] addresses = urlServer.substring(0, slash).split(",");
+      String[] addresses = urlServer.split(",");
       StringBuilder hosts = new StringBuilder();
       StringBuilder ports = new StringBuilder();
       for (String address : addresses) {
         int portIdx = address.lastIndexOf(':');
         if (portIdx != -1 && address.lastIndexOf(']') < portIdx) {
           String portStr = address.substring(portIdx + 1);
-          try {
-            int port = Integer.parseInt(portStr);
-            if (port < 1 || port > 65535) {
-              LOGGER.log(Level.WARNING, "JDBC URL port: {0} not valid (1:65535) ", portStr);
-              return null;
-            }
-          } catch (NumberFormatException ignore) {
-            LOGGER.log(Level.WARNING, "JDBC URL invalid port number: {0}", portStr);
-            return null;
-          }
           ports.append(portStr);
-          hosts.append(address.subSequence(0, portIdx));
+          CharSequence hostStr = address.subSequence(0, portIdx);
+          if (hostStr.length() == 0) {
+            hosts.append(PGProperty.PG_HOST.getDefaultValue());
+          } else {
+            hosts.append(hostStr);
+          }
         } else {
-          ports.append(DEFAULT_PORT);
+          ports.append(PGProperty.PG_PORT.getDefaultValue());
           hosts.append(address);
         }
         ports.append(',');
@@ -604,39 +556,98 @@ public class Driver implements java.sql.Driver {
       }
       ports.setLength(ports.length() - 1);
       hosts.setLength(hosts.length() - 1);
-      urlProps.setProperty("PGPORT", ports.toString());
-      urlProps.setProperty("PGHOST", hosts.toString());
+      PGProperty.PG_HOST.set(priority1Url, hosts.toString());
+      PGProperty.PG_PORT.set(priority1Url, ports.toString());
+    } else if (urlServer.startsWith("/")) {
+      return null;
     } else {
-      /*
-       if there are no defaults set or any one of PORT, HOST, DBNAME not set
-       then set it to default
-      */
-      if (defaults == null || !defaults.containsKey("PGPORT")) {
-        urlProps.setProperty("PGPORT", DEFAULT_PORT);
-      }
-      if (defaults == null || !defaults.containsKey("PGHOST")) {
-        urlProps.setProperty("PGHOST", "localhost");
-      }
-      if (defaults == null || !defaults.containsKey("PGDBNAME")) {
-        urlProps.setProperty("PGDBNAME", URLCoder.decode(urlServer));
+      String value = urlDecode(urlServer);
+      if (value == null) {
+        return null;
       }
+      priority1Url.setProperty(PGProperty.PG_DBNAME.getName(), value);
     }
 
     // parse the args part of the url
     String[] args = urlArgs.split("&");
+    String serviceName = null;
     for (String token : args) {
       if (token.isEmpty()) {
         continue;
       }
       int pos = token.indexOf('=');
       if (pos == -1) {
-        urlProps.setProperty(token, "");
+        priority1Url.setProperty(token, "");
       } else {
-        urlProps.setProperty(token.substring(0, pos), URLCoder.decode(token.substring(pos + 1)));
+        String pName = PGPropertyUtil.translatePGServiceToPGProperty(token.substring(0, pos));
+        String pValue = urlDecode(token.substring(pos + 1));
+        if (pValue == null) {
+          return null;
+        }
+        if (PGProperty.SERVICE.getName().equals(pName)) {
+          serviceName = pValue;
+        } else {
+          priority1Url.setProperty(pName, pValue);
+        }
       }
     }
 
-    return urlProps;
+    // load pg_service.conf
+    if (serviceName != null) {
+      LOGGER.log(Level.FINE, "Processing option [?service={0}]", serviceName);
+      Properties result = PGPropertyServiceParser.getServiceProperties(serviceName);
+      if (result == null) {
+        LOGGER.log(Level.WARNING, "Definition of service [{0}] not found", serviceName);
+        return null;
+      }
+      priority3Service.putAll(result);
+    }
+
+    // combine result based on order of priority
+    Properties result = new Properties();
+    result.putAll(priority1Url);
+    if (defaults != null) {
+      // priority 2 - forEach() returns all entries EXCEPT defaults
+      defaults.forEach(result::putIfAbsent);
+    }
+    priority3Service.forEach(result::putIfAbsent);
+    if (defaults != null) {
+      // priority 4 - stringPropertyNames() returns all entries INCLUDING defaults
+      defaults.stringPropertyNames().forEach(s -> result.putIfAbsent(s, castNonNull(defaults.getProperty(s))));
+    }
+    // priority 5 - PGProperty defaults for PGHOST, PGPORT, PGDBNAME
+    result.putIfAbsent(PGProperty.PG_PORT.getName(), castNonNull(PGProperty.PG_PORT.getDefaultValue()));
+    result.putIfAbsent(PGProperty.PG_HOST.getName(), castNonNull(PGProperty.PG_HOST.getDefaultValue()));
+    if (PGProperty.USER.get(result) != null) {
+      result.putIfAbsent(PGProperty.PG_DBNAME.getName(), castNonNull(PGProperty.USER.get(result)));
+    }
+
+    // consistency check
+    if (!PGPropertyUtil.propertiesConsistencyCheck(result)) {
+      return null;
+    }
+
+    // try to load .pgpass if password is missing
+    if (PGProperty.PASSWORD.get(result) == null) {
+      String password = PGPropertyPasswordParser.getPassword(
+          PGProperty.PG_HOST.get(result), PGProperty.PG_PORT.get(result), PGProperty.PG_DBNAME.get(result), PGProperty.USER.get(result)
+      );
+      if (password != null && !password.isEmpty()) {
+        PGProperty.PASSWORD.set(result, password);
+      }
+    }
+    //
+    return result;
+  }
+
+  // decode url, on failure log and return null
+  private static /* @Nullable */ String urlDecode(String url) {
+    try {
+      return URLCoder.decode(url);
+    } catch (IllegalArgumentException e) {
+      LOGGER.log(Level.FINE, "Url [{0}] parsing failed with error [{1}]", new Object[]{url, e.getMessage()});
+    }
+    return null;
   }
 
   /**
@@ -645,27 +656,14 @@ public class Driver implements java.sql.Driver {
   private static HostSpec[] hostSpecs(Properties props) {
     String[] hosts = castNonNull(props.getProperty("PGHOST")).split(",");
     String[] ports = castNonNull(props.getProperty("PGPORT")).split(",");
+    String localSocketAddress = props.getProperty("localSocketAddress");
     HostSpec[] hostSpecs = new HostSpec[hosts.length];
     for (int i = 0; i < hostSpecs.length; ++i) {
-      hostSpecs[i] = new HostSpec(hosts[i], Integer.parseInt(ports[i]));
+      hostSpecs[i] = new HostSpec(hosts[i], Integer.parseInt(ports[i]), localSocketAddress);
     }
     return hostSpecs;
   }
 
-  /**
-   * @return the username of the URL
-   */
-  private static String user(Properties props) {
-    return props.getProperty("user", "");
-  }
-
-  /**
-   * @return the database name of the URL
-   */
-  private static String database(Properties props) {
-    return props.getProperty("PGDBNAME", "");
-  }
-
   /**
    * @return the timeout from the URL, in milliseconds
    */
@@ -699,9 +697,7 @@ public class Driver implements java.sql.Driver {
         PSQLState.NOT_IMPLEMENTED.getState());
   }
 
-  //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.1"
   @Override
-  //#endif
   public java.util.logging.Logger getParentLogger() {
     return PARENT_LOGGER;
   }
