@@ -24,6 +24,37 @@ import java.util.Properties;
  */
 public enum PGProperty {
 
+  /**
+   * Specifies if number of rows, used during fetching rows of a result set, should be computed
+   * dynamically. Number of rows will be calculated by dividing maxResultBuffer size by max row size
+   * observed so far, rounded down. First fetch will have number of rows declared in
+   * defaultRowFetchSize. Number of rows can be limited by adaptiveFetchMinimum and
+   * adaptiveFetchMaximum. Requires declaring of maxResultBuffer and defaultRowFetchSize to work.
+   * Default value is false.
+   */
+  ADAPTIVE_FETCH(
+      "adaptiveFetch",
+      "false",
+      "Specifies if number of rows fetched in ResultSet should be adaptive to maxResultBuffer and max row size."),
+
+  /**
+   * Specifies the highest number of rows which can be calculated by adaptiveFetch. Requires
+   * adaptiveFetch set to true to work. Default value is -1 (used as infinity).
+   */
+  ADAPTIVE_FETCH_MAXIMUM(
+      "adaptiveFetchMaximum",
+      "-1",
+      "Specifies maximum number of rows used by adaptive fetch."),
+
+  /**
+   * Specifies the lowest number of rows which can be calculated by adaptiveFetch. Requires
+   * adaptiveFetch set to true to work. Default value is 0.
+   */
+  ADAPTIVE_FETCH_MINIMUM(
+      "adaptiveFetchMinimum",
+      "0",
+      "Specifies minimum number of rows used by adaptive fetch."),
+
   /**
    * When using the V3 protocol the driver monitors changes in certain server configuration
    * parameters that should not be touched by end users. The {@code client_encoding} setting is set
@@ -31,25 +62,35 @@ public enum PGProperty {
    * connection.
    */
   ALLOW_ENCODING_CHANGES(
-    "allowEncodingChanges",
-    "false",
-    "Allow for changes in client_encoding"),
+      "allowEncodingChanges",
+      "false",
+      "Allow for changes in client_encoding"),
 
   /**
    * The application name (require server version &gt;= 9.0).
    */
   APPLICATION_NAME(
-    "ApplicationName",
-    DriverInfo.DRIVER_NAME,
-    "Name of the Application (backend >= 9.0)"),
+      "ApplicationName",
+      DriverInfo.DRIVER_NAME,
+      "Name of the Application (backend >= 9.0)"),
 
   /**
    * Assume the server is at least that version.
    */
   ASSUME_MIN_SERVER_VERSION(
-    "assumeMinServerVersion",
-    null,
-    "Assume the server is at least that version"),
+      "assumeMinServerVersion",
+      null,
+      "Assume the server is at least that version"),
+
+  /**
+   * AuthenticationPluginClass
+   */
+
+  AUTHENTICATION_PLUGIN_CLASS_NAME(
+      "authenticationPluginClassName",
+      null,
+      "Name of class which implements AuthenticationPlugin"
+  ),
 
   /**
    * Specifies what the driver should do if a query fails. In {@code autosave=always} mode, JDBC driver sets a savepoint before each query,
@@ -58,39 +99,39 @@ public enum PGProperty {
    * like 'cached statement cannot change return type' or 'statement XXX is not valid' so JDBC driver rollsback and retries
    */
   AUTOSAVE(
-    "autosave",
-    "never",
-    "Specifies what the driver should do if a query fails. In autosave=always mode, JDBC driver sets a savepoint before each query, "
-        + "and rolls back to that savepoint in case of failure. In autosave=never mode (default), no savepoint dance is made ever. "
-        + "In autosave=conservative mode, safepoint is set for each query, however the rollback is done only for rare cases"
-        + " like 'cached statement cannot change return type' or 'statement XXX is not valid' so JDBC driver rollsback and retries",
-    false,
-    new String[] {"always", "never", "conservative"}),
+      "autosave",
+      "never",
+      "Specifies what the driver should do if a query fails. In autosave=always mode, JDBC driver sets a savepoint before each query, "
+          + "and rolls back to that savepoint in case of failure. In autosave=never mode (default), no savepoint dance is made ever. "
+          + "In autosave=conservative mode, safepoint is set for each query, however the rollback is done only for rare cases"
+          + " like 'cached statement cannot change return type' or 'statement XXX is not valid' so JDBC driver rollsback and retries",
+      false,
+      new String[] {"always", "never", "conservative"}),
 
   /**
    * Use binary format for sending and receiving data if possible.
    */
   BINARY_TRANSFER(
-    "binaryTransfer",
-    "true",
-    "Use binary format for sending and receiving data if possible"),
+      "binaryTransfer",
+      "true",
+      "Use binary format for sending and receiving data if possible"),
 
   /**
    * Comma separated list of types to disable binary transfer. Either OID numbers or names.
    * Overrides values in the driver default set and values set with binaryTransferEnable.
    */
   BINARY_TRANSFER_DISABLE(
-    "binaryTransferDisable",
-    "",
-    "Comma separated list of types to disable binary transfer. Either OID numbers or names. Overrides values in the driver default set and values set with binaryTransferEnable."),
+      "binaryTransferDisable",
+      "",
+      "Comma separated list of types to disable binary transfer. Either OID numbers or names. Overrides values in the driver default set and values set with binaryTransferEnable."),
 
   /**
    * Comma separated list of types to enable binary transfer. Either OID numbers or names
    */
   BINARY_TRANSFER_ENABLE(
-    "binaryTransferEnable",
-    "",
-    "Comma separated list of types to enable binary transfer. Either OID numbers or names"),
+      "binaryTransferEnable",
+      "",
+      "Comma separated list of types to enable binary transfer. Either OID numbers or names"),
 
   /**
    * Cancel command is sent out of band over its own connection, so cancel message can itself get
@@ -99,19 +140,19 @@ public enum PGProperty {
    * The timeout is specified in seconds. Default value is 10 seconds.
    */
   CANCEL_SIGNAL_TIMEOUT(
-    "cancelSignalTimeout",
-    "10",
-    "The timeout that is used for sending cancel command."),
+      "cancelSignalTimeout",
+      "10",
+      "The timeout that is used for sending cancel command."),
 
   /**
    * Determine whether SAVEPOINTS used in AUTOSAVE will be released per query or not
    */
   CLEANUP_SAVEPOINTS(
-    "cleanupSavepoints",
-    "false",
-    "Determine whether SAVEPOINTS used in AUTOSAVE will be released per query or not",
-    false,
-    new String[] {"true", "false"}),
+      "cleanupSavepoints",
+      "false",
+      "Determine whether SAVEPOINTS used in AUTOSAVE will be released per query or not",
+      false,
+      new String[] {"true", "false"}),
 
   /**
    * <p>The timeout value used for socket connect operations. If connecting to the server takes longer
@@ -120,51 +161,51 @@ public enum PGProperty {
    * <p>The timeout is specified in seconds and a value of zero means that it is disabled.</p>
    */
   CONNECT_TIMEOUT(
-    "connectTimeout",
-    "10",
-    "The timeout value used for socket connect operations."),
+      "connectTimeout",
+      "10",
+      "The timeout value used for socket connect operations."),
 
   /**
    * Specify the schema (or several schema separated by commas) to be set in the search-path. This schema will be used to resolve
    * unqualified object names used in statements over this connection.
    */
   CURRENT_SCHEMA(
-    "currentSchema",
-    null,
-    "Specify the schema (or several schema separated by commas) to be set in the search-path"),
+      "currentSchema",
+      null,
+      "Specify the schema (or several schema separated by commas) to be set in the search-path"),
 
   /**
    * Specifies the maximum number of fields to be cached per connection. A value of {@code 0} disables the cache.
    */
   DATABASE_METADATA_CACHE_FIELDS(
-    "databaseMetadataCacheFields",
-    "65536",
-    "Specifies the maximum number of fields to be cached per connection. A value of {@code 0} disables the cache."),
+      "databaseMetadataCacheFields",
+      "65536",
+      "Specifies the maximum number of fields to be cached per connection. A value of {@code 0} disables the cache."),
 
   /**
    * Specifies the maximum size (in megabytes) of fields to be cached per connection. A value of {@code 0} disables the cache.
    */
   DATABASE_METADATA_CACHE_FIELDS_MIB(
-    "databaseMetadataCacheFieldsMiB",
-    "5",
-    "Specifies the maximum size (in megabytes) of fields to be cached per connection. A value of {@code 0} disables the cache."),
+      "databaseMetadataCacheFieldsMiB",
+      "5",
+      "Specifies the maximum size (in megabytes) of fields to be cached per connection. A value of {@code 0} disables the cache."),
 
   /**
    * Default parameter for {@link java.sql.Statement#getFetchSize()}. A value of {@code 0} means
    * that need fetch all rows at once
    */
   DEFAULT_ROW_FETCH_SIZE(
-    "defaultRowFetchSize",
-    "0",
-    "Positive number of rows that should be fetched from the database when more rows are needed for ResultSet by each fetch iteration"),
+      "defaultRowFetchSize",
+      "0",
+      "Positive number of rows that should be fetched from the database when more rows are needed for ResultSet by each fetch iteration"),
 
   /**
    * Enable optimization that disables column name sanitiser.
    */
   DISABLE_COLUMN_SANITISER(
-    "disableColumnSanitiser",
-    "false",
-    "Enable optimization that disables column name sanitiser"),
+      "disableColumnSanitiser",
+      "false",
+      "Enable optimization that disables column name sanitiser"),
 
   /**
    * Specifies how the driver transforms JDBC escape call syntax into underlying SQL, for invoking procedures or functions. (backend &gt;= 11)
@@ -173,21 +214,21 @@ public enum PGProperty {
    * In {@code escapeSyntaxCallMode=call} mode, the driver always uses a CALL statement (allowing procedure invocation only).
    */
   ESCAPE_SYNTAX_CALL_MODE(
-    "escapeSyntaxCallMode",
-    "select",
-    "Specifies how the driver transforms JDBC escape call syntax into underlying SQL, for invoking procedures or functions. (backend >= 11)"
-      + "In escapeSyntaxCallMode=select mode (the default), the driver always uses a SELECT statement (allowing function invocation only)."
-      + "In escapeSyntaxCallMode=callIfNoReturn mode, the driver uses a CALL statement (allowing procedure invocation) if there is no return parameter specified, otherwise the driver uses a SELECT statement."
-      + "In escapeSyntaxCallMode=call mode, the driver always uses a CALL statement (allowing procedure invocation only).",
-    false,
-    new String[] {"select", "callIfNoReturn", "call"}),
+      "escapeSyntaxCallMode",
+      "select",
+      "Specifies how the driver transforms JDBC escape call syntax into underlying SQL, for invoking procedures or functions. (backend >= 11)"
+          + "In escapeSyntaxCallMode=select mode (the default), the driver always uses a SELECT statement (allowing function invocation only)."
+          + "In escapeSyntaxCallMode=callIfNoReturn mode, the driver uses a CALL statement (allowing procedure invocation) if there is no return parameter specified, otherwise the driver uses a SELECT statement."
+          + "In escapeSyntaxCallMode=call mode, the driver always uses a CALL statement (allowing procedure invocation only).",
+      false,
+      new String[] {"select", "callIfNoReturn", "call"}),
 
   GSS_ENC_MODE(
       "gssEncMode",
-      "prefer",
+      "allow",
       "Force Encoded GSS Mode",
       false,
-      new String[] {"disable", "prefer", "require"}
+      new String[] {"disable", "allow", "prefer", "require"}
   ),
 
   /**
@@ -199,33 +240,33 @@ public enum PGProperty {
    * to be used when the server requests Kerberos or SSPI authentication.
    */
   GSS_LIB(
-    "gsslib",
-    "auto",
-    "Force SSSPI or GSSAPI",
-    false,
-    new String[] {"auto", "sspi", "gssapi"}),
+      "gsslib",
+      "auto",
+      "Force SSSPI or GSSAPI",
+      false,
+      new String[] {"auto", "sspi", "gssapi"}),
 
   /**
    * Enable mode to filter out the names of database objects for which the current user has no privileges
    * granted from appearing in the DatabaseMetaData returned by the driver.
    */
   HIDE_UNPRIVILEGED_OBJECTS(
-    "hideUnprivilegedObjects",
-    "false",
-    "Enable hiding of database objects for which the current user has no privileges granted from the DatabaseMetaData"),
+      "hideUnprivilegedObjects",
+      "false",
+      "Enable hiding of database objects for which the current user has no privileges granted from the DatabaseMetaData"),
 
   HOST_RECHECK_SECONDS(
-    "hostRecheckSeconds",
-    "10",
-    "Specifies period (seconds) after which the host status is checked again in case it has changed"),
+      "hostRecheckSeconds",
+      "10",
+      "Specifies period (seconds) after which the host status is checked again in case it has changed"),
 
   /**
    * Specifies the name of the JAAS system or application login configuration.
    */
   JAAS_APPLICATION_NAME(
-    "jaasApplicationName",
-    null,
-    "Specifies the name of the JAAS system or application login configuration."),
+      "jaasApplicationName",
+      "pgjdbc",
+      "Specifies the name of the JAAS system or application login configuration."),
 
   /**
    * Flag to enable/disable obtaining a GSS credential via JAAS login before authenticating.
@@ -233,136 +274,122 @@ public enum PGProperty {
    * or using native GSS with system property sun.security.jgss.native=true
    */
   JAAS_LOGIN(
-    "jaasLogin",
-    "true",
-    "Login with JAAS before doing GSSAPI authentication"),
+      "jaasLogin",
+      "true",
+      "Login with JAAS before doing GSSAPI authentication"),
 
   /**
    * The Kerberos service name to use when authenticating with GSSAPI. This is equivalent to libpq's
    * PGKRBSRVNAME environment variable.
    */
   KERBEROS_SERVER_NAME(
-    "kerberosServerName",
-    null,
-    "The Kerberos service name to use when authenticating with GSSAPI."),
+      "kerberosServerName",
+      null,
+      "The Kerberos service name to use when authenticating with GSSAPI."),
 
   LOAD_BALANCE_HOSTS(
-    "loadBalanceHosts",
-    "false",
-    "If disabled hosts are connected in the given order. If enabled hosts are chosen randomly from the set of suitable candidates"),
+      "loadBalanceHosts",
+      "false",
+      "If disabled hosts are connected in the given order. If enabled hosts are chosen randomly from the set of suitable candidates"),
 
   /**
-   * <p>File name output of the Logger, if set, the Logger will use a
-   * {@link java.util.logging.FileHandler} to write to a specified file. If the parameter is not set
-   * or the file can't be created the {@link java.util.logging.ConsoleHandler} will be used instead.</p>
-   *
-   * <p>Parameter should be use together with {@link PGProperty#LOGGER_LEVEL}</p>
+   * This property is no longer used by the driver and will be ignored.
+   * Logging is configured via java.util.logging.
    */
   LOGGER_FILE(
-    "loggerFile",
-    null,
-    "File name output of the Logger"),
+      "loggerFile",
+      null,
+      "File name output of the Logger"),
 
   /**
-   * <p>Logger level of the driver. Allowed values: {@code OFF}, {@code DEBUG} or {@code TRACE}.</p>
-   *
-   * <p>This enable the {@link java.util.logging.Logger} of the driver based on the following mapping
-   * of levels:</p>
-   * <ul>
-   *     <li>FINE -&gt; DEBUG</li>
-   *     <li>FINEST -&gt; TRACE</li>
-   * </ul>
-   *
-   * <p><b>NOTE:</b> The recommended approach to enable java.util.logging is using a
-   * {@code logging.properties} configuration file with the property
-   * {@code -Djava.util.logging.config.file=myfile} or if your are using an application server
-   * you should use the appropriate logging subsystem.</p>
+   * This property is no longer used by the driver and will be ignored.
+   * Logging is configured via java.util.logging.
    */
   LOGGER_LEVEL(
-    "loggerLevel",
-    null,
-    "Logger level of the driver",
-    false,
-    new String[] {"OFF", "DEBUG", "TRACE"}),
+      "loggerLevel",
+      null,
+      "Logger level of the driver",
+      false,
+      new String[] {"OFF", "DEBUG", "TRACE"}),
 
   /**
    * Specify how long to wait for establishment of a database connection. The timeout is specified
    * in seconds.
    */
   LOGIN_TIMEOUT(
-    "loginTimeout",
-    "0",
-    "Specify how long to wait for establishment of a database connection."),
+      "loginTimeout",
+      "0",
+      "Specify how long to wait for establishment of a database connection."),
 
   /**
    * Whether to include full server error detail in exception messages.
    */
   LOG_SERVER_ERROR_DETAIL(
-    "logServerErrorDetail",
-    "true",
-    "Include full server error detail in exception messages. If disabled then only the error itself will be included."),
+      "logServerErrorDetail",
+      "true",
+      "Include full server error detail in exception messages. If disabled then only the error itself will be included."),
 
   /**
    * When connections that are not explicitly closed are garbage collected, log the stacktrace from
    * the opening of the connection to trace the leak source.
    */
   LOG_UNCLOSED_CONNECTIONS(
-    "logUnclosedConnections",
-    "false",
-    "When connections that are not explicitly closed are garbage collected, log the stacktrace from the opening of the connection to trace the leak source"),
+      "logUnclosedConnections",
+      "false",
+      "When connections that are not explicitly closed are garbage collected, log the stacktrace from the opening of the connection to trace the leak source"),
 
   /**
    * Specifies size of buffer during fetching result set. Can be specified as specified size or
    * percent of heap memory.
    */
   MAX_RESULT_BUFFER(
-    "maxResultBuffer",
-    null,
-    "Specifies size of buffer during fetching result set. Can be specified as specified size or percent of heap memory."),
+      "maxResultBuffer",
+      null,
+      "Specifies size of buffer during fetching result set. Can be specified as specified size or percent of heap memory."),
 
   /**
    * Specify 'options' connection initialization parameter.
    * The value of this parameter may contain spaces and other special characters or their URL representation.
    */
   OPTIONS(
-    "options",
-    null,
-    "Specify 'options' connection initialization parameter."),
+      "options",
+      null,
+      "Specify 'options' connection initialization parameter."),
 
   /**
    * Password to use when authenticating.
    */
   PASSWORD(
-    "password",
-    null,
-    "Password to use when authenticating.",
-    false),
+      "password",
+      null,
+      "Password to use when authenticating.",
+      false),
 
   /**
    * Database name to connect to (may be specified directly in the JDBC URL).
    */
   PG_DBNAME(
-    "PGDBNAME",
-    null,
-    "Database name to connect to (may be specified directly in the JDBC URL)",
-    true),
+      "PGDBNAME",
+      null,
+      "Database name to connect to (may be specified directly in the JDBC URL)",
+      true),
 
   /**
    * Hostname of the PostgreSQL server (may be specified directly in the JDBC URL).
    */
   PG_HOST(
-    "PGHOST",
-    null,
-    "Hostname of the PostgreSQL server (may be specified directly in the JDBC URL)",
-    false),
+      "PGHOST",
+      "localhost",
+      "Hostname of the PostgreSQL server (may be specified directly in the JDBC URL)",
+      false),
 
   /**
    * Port of the PostgreSQL server (may be specified directly in the JDBC URL).
    */
   PG_PORT(
-    "PGPORT",
-    null,
-    "Port of the PostgreSQL server (may be specified directly in the JDBC URL)"),
+      "PGPORT",
+      "5432",
+      "Port of the PostgreSQL server (may be specified directly in the JDBC URL)"),
 
   /**
    * <p>Specifies which mode is used to execute queries to database: simple means ('Q' execute, no parse, no bind, text mode only),
@@ -372,82 +399,93 @@ public enum PGProperty {
    * <p>This mode is meant for debugging purposes and/or for cases when extended protocol cannot be used (e.g. logical replication protocol)</p>
    */
   PREFER_QUERY_MODE(
-    "preferQueryMode",
-    "extended",
-    "Specifies which mode is used to execute queries to database: simple means ('Q' execute, no parse, no bind, text mode only), "
-        + "extended means always use bind/execute messages, extendedForPrepared means extended for prepared statements only, "
-        + "extendedCacheEverything means use extended protocol and try cache every statement (including Statement.execute(String sql)) in a query cache.", false,
-    new String[] {"extended", "extendedForPrepared", "extendedCacheEverything", "simple"}),
+      "preferQueryMode",
+      "extended",
+      "Specifies which mode is used to execute queries to database: simple means ('Q' execute, no parse, no bind, text mode only), "
+          + "extended means always use bind/execute messages, extendedForPrepared means extended for prepared statements only, "
+          + "extendedCacheEverything means use extended protocol and try cache every statement (including Statement.execute(String sql)) in a query cache.", false,
+      new String[] {"extended", "extendedForPrepared", "extendedCacheEverything", "simple"}),
 
   /**
    * Specifies the maximum number of entries in cache of prepared statements. A value of {@code 0}
    * disables the cache.
    */
   PREPARED_STATEMENT_CACHE_QUERIES(
-    "preparedStatementCacheQueries",
-    "256",
-    "Specifies the maximum number of entries in per-connection cache of prepared statements. A value of {@code 0} disables the cache."),
+      "preparedStatementCacheQueries",
+      "256",
+      "Specifies the maximum number of entries in per-connection cache of prepared statements. A value of {@code 0} disables the cache."),
 
   /**
    * Specifies the maximum size (in megabytes) of the prepared statement cache. A value of {@code 0}
    * disables the cache.
    */
   PREPARED_STATEMENT_CACHE_SIZE_MIB(
-    "preparedStatementCacheSizeMiB",
-    "5",
-    "Specifies the maximum size (in megabytes) of a per-connection prepared statement cache. A value of {@code 0} disables the cache."),
+      "preparedStatementCacheSizeMiB",
+      "5",
+      "Specifies the maximum size (in megabytes) of a per-connection prepared statement cache. A value of {@code 0} disables the cache."),
 
   /**
    * Sets the default threshold for enabling server-side prepare. A value of {@code -1} stands for
    * forceBinary
    */
   PREPARE_THRESHOLD(
-    "prepareThreshold",
-    "5",
-    "Statement prepare threshold. A value of {@code -1} stands for forceBinary"),
+      "prepareThreshold",
+      "5",
+      "Statement prepare threshold. A value of {@code -1} stands for forceBinary"),
 
   /**
    * Force use of a particular protocol version when connecting, if set, disables protocol version
    * fallback.
    */
   PROTOCOL_VERSION(
-    "protocolVersion",
-    null,
-    "Force use of a particular protocol version when connecting, currently only version 3 is supported.",
-    false,
-    new String[] {"3"}),
+      "protocolVersion",
+      null,
+      "Force use of a particular protocol version when connecting, currently only version 3 is supported.",
+      false,
+      new String[] {"3"}),
 
+  /**
+   * Quote returning columns.
+   * There are some ORM's that quote everything, including returning columns
+   * If we quote them, then we end up sending ""colname"" to the backend
+   * which will not be found
+   */
+  QUOTE_RETURNING_IDENTIFIERS(
+    "quoteReturningIdentifiers",
+    "true",
+    "Quote identifiers provided in returning array",
+      false),
   /**
    * Puts this connection in read-only mode.
    */
   READ_ONLY(
-    "readOnly",
-    "false",
-    "Puts this connection in read-only mode"),
+      "readOnly",
+      "false",
+      "Puts this connection in read-only mode"),
 
   /**
    * Connection parameter to control behavior when
    * {@link Connection#setReadOnly(boolean)} is set to {@code true}.
    */
   READ_ONLY_MODE(
-    "readOnlyMode",
-    "transaction",
-    "Controls the behavior when a connection is set to be read only, one of 'ignore', 'transaction', or 'always' "
-      + "When 'ignore', setting readOnly has no effect. "
-      + "When 'transaction' setting readOnly to 'true' will cause transactions to BEGIN READ ONLY if autocommit is 'false'. "
-      + "When 'always' setting readOnly to 'true' will set the session to READ ONLY if autoCommit is 'true' "
-      + "and the transaction to BEGIN READ ONLY if autocommit is 'false'.",
-    false,
-    new String[] {"ignore", "transaction", "always"}),
+      "readOnlyMode",
+      "transaction",
+      "Controls the behavior when a connection is set to be read only, one of 'ignore', 'transaction', or 'always' "
+          + "When 'ignore', setting readOnly has no effect. "
+          + "When 'transaction' setting readOnly to 'true' will cause transactions to BEGIN READ ONLY if autocommit is 'false'. "
+          + "When 'always' setting readOnly to 'true' will set the session to READ ONLY if autoCommit is 'true' "
+          + "and the transaction to BEGIN READ ONLY if autocommit is 'false'.",
+      false,
+      new String[] {"ignore", "transaction", "always"}),
 
   /**
    * Socket read buffer size (SO_RECVBUF). A value of {@code -1}, which is the default, means system
    * default.
    */
   RECEIVE_BUFFER_SIZE(
-    "receiveBufferSize",
-    "-1",
-    "Socket read buffer size"),
+      "receiveBufferSize",
+      "-1",
+      "Socket read buffer size"),
 
   /**
    * <p>Connection parameter passed in the startup message. This parameter accepts two values; "true"
@@ -460,42 +498,52 @@ public enum PGProperty {
    * parameter &gt;= 9.4 (backend &gt;= 9.4)</p>
    */
   REPLICATION(
-    "replication",
-    null,
-    "Connection parameter passed in startup message, one of 'true' or 'database' "
-      + "Passing 'true' tells the backend to go into walsender mode, "
-      + "wherein a small set of replication commands can be issued instead of SQL statements. "
-      + "Only the simple query protocol can be used in walsender mode. "
-      + "Passing 'database' as the value instructs walsender to connect "
-      + "to the database specified in the dbname parameter, "
-      + "which will allow the connection to be used for logical replication "
-      + "from that database. "
-      + "(backend >= 9.4)"),
+      "replication",
+      null,
+      "Connection parameter passed in startup message, one of 'true' or 'database' "
+          + "Passing 'true' tells the backend to go into walsender mode, "
+          + "wherein a small set of replication commands can be issued instead of SQL statements. "
+          + "Only the simple query protocol can be used in walsender mode. "
+          + "Passing 'database' as the value instructs walsender to connect "
+          + "to the database specified in the dbname parameter, "
+          + "which will allow the connection to be used for logical replication "
+          + "from that database. "
+          + "(backend >= 9.4)"),
 
   /**
    * Configure optimization to enable batch insert re-writing.
    */
   REWRITE_BATCHED_INSERTS(
-    "reWriteBatchedInserts",
-    "false",
-    "Enable optimization to rewrite and collapse compatible INSERT statements that are batched."),
+      "reWriteBatchedInserts",
+      "false",
+      "Enable optimization to rewrite and collapse compatible INSERT statements that are batched."),
 
   /**
    * Socket write buffer size (SO_SNDBUF). A value of {@code -1}, which is the default, means system
    * default.
    */
   SEND_BUFFER_SIZE(
-    "sendBufferSize",
-    "-1",
-    "Socket write buffer size"),
+      "sendBufferSize",
+      "-1",
+      "Socket write buffer size"),
+
+  /**
+   * Service name to use for additional parameters. It specifies a service name in "pg_service
+   * .conf" that holds additional connection parameters. This allows applications to specify only
+   * a service name so connection parameters can be centrally maintained.
+   */
+  SERVICE(
+      "service",
+      null,
+      "Service name to be searched in pg_service.conf resource"),
 
   /**
    * Socket factory used to create socket. A null value, which is the default, means system default.
    */
   SOCKET_FACTORY(
-    "socketFactory",
-    null,
-    "Specify a socket factory for socket creation"),
+      "socketFactory",
+      null,
+      "Specify a socket factory for socket creation"),
 
   /**
    * The String argument to give to the constructor of the Socket Factory.
@@ -503,9 +551,9 @@ public enum PGProperty {
    */
   @Deprecated
   SOCKET_FACTORY_ARG(
-    "socketFactoryArg",
-    null,
-    "Argument forwarded to constructor of SocketFactory class."),
+      "socketFactoryArg",
+      null,
+      "Argument forwarded to constructor of SocketFactory class."),
 
   /**
    * The timeout value used for socket read operations. If reading from the server takes longer than
@@ -514,34 +562,34 @@ public enum PGProperty {
    * value of zero means that it is disabled.
    */
   SOCKET_TIMEOUT(
-    "socketTimeout",
-    "0",
-    "The timeout value used for socket read operations."),
+      "socketTimeout",
+      "0",
+      "The timeout value used for socket read operations."),
 
   /**
    * Control use of SSL: empty or {@code true} values imply {@code sslmode==verify-full}
    */
   SSL(
-    "ssl",
-    null,
-    "Control use of SSL (any non-null value causes SSL to be required)"),
+      "ssl",
+      null,
+      "Control use of SSL (any non-null value causes SSL to be required)"),
 
   /**
    * File containing the SSL Certificate. Default will be the file {@code postgresql.crt} in {@code
    * $HOME/.postgresql} (*nix) or {@code %APPDATA%\postgresql} (windows).
    */
   SSL_CERT(
-    "sslcert",
-    null,
-    "The location of the client's SSL certificate"),
+      "sslcert",
+      null,
+      "The location of the client's SSL certificate"),
 
   /**
    * Classname of the SSL Factory to use (instance of {@code javax.net.ssl.SSLSocketFactory}).
    */
   SSL_FACTORY(
-    "sslfactory",
-    null,
-    "Provide a SSLSocketFactory class when using SSL."),
+      "sslfactory",
+      null,
+      "Provide a SSLSocketFactory class when using SSL."),
 
   /**
    * The String argument to give to the constructor of the SSL Factory.
@@ -549,27 +597,27 @@ public enum PGProperty {
    */
   @Deprecated
   SSL_FACTORY_ARG(
-    "sslfactoryarg",
-    null,
-    "Argument forwarded to constructor of SSLSocketFactory class."),
+      "sslfactoryarg",
+      null,
+      "Argument forwarded to constructor of SSLSocketFactory class."),
 
   /**
    * Classname of the SSL HostnameVerifier to use (instance of {@code
    * javax.net.ssl.HostnameVerifier}).
    */
   SSL_HOSTNAME_VERIFIER(
-    "sslhostnameverifier",
-    null,
-    "A class, implementing javax.net.ssl.HostnameVerifier that can verify the server"),
+      "sslhostnameverifier",
+      null,
+      "A class, implementing javax.net.ssl.HostnameVerifier that can verify the server"),
 
   /**
    * File containing the SSL Key. Default will be the file {@code postgresql.pk8} in {@code
    * $HOME/.postgresql} (*nix) or {@code %APPDATA%\postgresql} (windows).
    */
   SSL_KEY(
-    "sslkey",
-    null,
-    "The location of the client's PKCS#8 SSL key"),
+      "sslkey",
+      null,
+      "The location of the client's PKCS#8 SSL key"),
 
   /**
    * Parameter governing the use of SSL. The allowed values are {@code disable}, {@code allow},
@@ -578,28 +626,28 @@ public enum PGProperty {
    * Default mode is "require"
    */
   SSL_MODE(
-    "sslmode",
-    null,
-    "Parameter governing the use of SSL",
-    false,
-    new String[] {"disable", "allow", "prefer", "require", "verify-ca", "verify-full"}),
+      "sslmode",
+      null,
+      "Parameter governing the use of SSL",
+      false,
+      new String[] {"disable", "allow", "prefer", "require", "verify-ca", "verify-full"}),
 
   /**
    * The SSL password to use in the default CallbackHandler.
    */
   SSL_PASSWORD(
-    "sslpassword",
-    null,
-    "The password for the client's ssl key (ignored if sslpasswordcallback is set)"),
+      "sslpassword",
+      null,
+      "The password for the client's ssl key (ignored if sslpasswordcallback is set)"),
 
 
   /**
    * The classname instantiating {@code javax.security.auth.callback.CallbackHandler} to use.
    */
   SSL_PASSWORD_CALLBACK(
-    "sslpasswordcallback",
-    null,
-    "A class, implementing javax.security.auth.callback.CallbackHandler that can handle PassworCallback for the ssl password."),
+      "sslpasswordcallback",
+      null,
+      "A class, implementing javax.security.auth.callback.CallbackHandler that can handle PassworCallback for the ssl password."),
 
   /**
    * File containing the root certificate when validating server ({@code sslmode} = {@code
@@ -607,69 +655,74 @@ public enum PGProperty {
    * $HOME/.postgresql} (*nix) or {@code %APPDATA%\postgresql} (windows).
    */
   SSL_ROOT_CERT(
-    "sslrootcert",
-    null,
-    "The location of the root certificate for authenticating the server."),
+      "sslrootcert",
+      null,
+      "The location of the root certificate for authenticating the server."),
 
   /**
    * Specifies the name of the SSPI service class that forms the service class part of the SPN. The
    * default, {@code POSTGRES}, is almost always correct.
    */
   SSPI_SERVICE_CLASS(
-    "sspiServiceClass",
-    "POSTGRES",
-    "The Windows SSPI service class for SPN"),
+      "sspiServiceClass",
+      "POSTGRES",
+      "The Windows SSPI service class for SPN"),
 
   /**
    * Bind String to either {@code unspecified} or {@code varchar}. Default is {@code varchar} for
    * 8.0+ backends.
    */
   STRING_TYPE(
-    "stringtype",
-    null,
-    "The type to bind String parameters as (usually 'varchar', 'unspecified' allows implicit casting to other types)",
-    false,
-    new String[] {"unspecified", "varchar"}),
+      "stringtype",
+      null,
+      "The type to bind String parameters as (usually 'varchar', 'unspecified' allows implicit casting to other types)",
+      false,
+      new String[] {"unspecified", "varchar"}),
 
   TARGET_SERVER_TYPE(
-    "targetServerType",
-    "any",
-    "Specifies what kind of server to connect",
-    false,
-    new String [] {"any", "primary", "master", "slave", "secondary",  "preferSlave", "preferSecondary"}),
+      "targetServerType",
+      "any",
+      "Specifies what kind of server to connect",
+      false,
+      new String [] {"any", "primary", "master", "slave", "secondary",  "preferSlave", "preferSecondary"}),
 
   /**
    * Enable or disable TCP keep-alive. The default is {@code false}.
    */
   TCP_KEEP_ALIVE(
-    "tcpKeepAlive",
-    "false",
-    "Enable or disable TCP keep-alive. The default is {@code false}."),
+      "tcpKeepAlive",
+      "false",
+      "Enable or disable TCP keep-alive. The default is {@code false}."),
 
+  TCP_NO_DELAY(
+      "tcpNoDelay",
+      "false",
+      "Enable or disable TCP no delay. The default is (@code false}."
+  ),
   /**
    * Specifies the length to return for types of unknown length.
    */
   UNKNOWN_LENGTH(
-    "unknownLength",
-    Integer.toString(Integer.MAX_VALUE),
-    "Specifies the length to return for types of unknown length"),
+      "unknownLength",
+      Integer.toString(Integer.MAX_VALUE),
+      "Specifies the length to return for types of unknown length"),
 
   /**
    * Username to connect to the database as.
    */
   USER(
-    "user",
-    null,
-    "Username to connect to the database as.",
-    true),
+      "user",
+      null,
+      "Username to connect to the database as.",
+      true),
 
   /**
    * Use SPNEGO in SSPI authentication requests.
    */
   USE_SPNEGO(
-    "useSpnego",
-    "false",
-    "Use SPNEGO in SSPI authentication requests"),
+      "useSpnego",
+      "false",
+      "Use SPNEGO in SSPI authentication requests"),
 
   /**
    * Factory class to instantiate factories for XML processing.
@@ -678,9 +731,9 @@ public enum PGProperty {
    * Or specify a custom class that implements {@code org.postgresql.xml.PGXmlFactoryFactory}.
    */
   XML_FACTORY_FACTORY(
-    "xmlFactoryFactory",
-    "",
-    "Factory class to instantiate factories for XML processing"),
+      "xmlFactoryFactory",
+      "",
+      "Factory class to instantiate factories for XML processing"),
 
   ;
 
@@ -714,6 +767,7 @@ public enum PGProperty {
   }
 
   private static final Map<String, PGProperty> PROPS_BY_NAME = new HashMap<String, PGProperty>();
+
   static {
     for (PGProperty prop : PGProperty.values()) {
       if (PROPS_BY_NAME.put(prop.getName(), prop) != null) {
