@@ -26,6 +26,7 @@ import org.postgresql.hostchooser.HostRequirement;
 import org.postgresql.hostchooser.HostStatus;
 import org.postgresql.jdbc.GSSEncMode;
 import org.postgresql.jdbc.SslMode;
+import org.postgresql.plugin.AuthenticationRequestType;
 import org.postgresql.sspi.ISSPIClient;
 import org.postgresql.util.GT;
 import org.postgresql.util.HostSpec;
@@ -38,6 +39,7 @@ import org.postgresql.util.ServerErrorMessage;
 
 import java.io.IOException;
 import java.net.ConnectException;
+import java.nio.charset.StandardCharsets;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.HashMap;
@@ -74,6 +76,8 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
   private static final int AUTH_REQ_SASL_CONTINUE = 11;
   private static final int AUTH_REQ_SASL_FINAL = 12;
 
+  private static final String IN_HOT_STANDBY = "in_hot_standby";
+
   private ISSPIClient createSSPI(PGStream pgStream,
       /* @Nullable */ String spnServiceClass,
       boolean enableNegotiate) {
@@ -89,83 +93,101 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
     }
   }
 
-  private PGStream tryConnect(String user, String database,
-      Properties info, SocketFactory socketFactory, HostSpec hostSpec,
+  private PGStream tryConnect(Properties info, SocketFactory socketFactory, HostSpec hostSpec,
       SslMode sslMode, GSSEncMode gssEncMode)
       throws SQLException, IOException {
     int connectTimeout = PGProperty.CONNECT_TIMEOUT.getInt(info) * 1000;
+    String user = PGProperty.USER.get(info);
+    String database = PGProperty.PG_DBNAME.get(info);
+    if (user == null) {
+      throw new PSQLException(GT.tr("User cannot be null"), PSQLState.INVALID_NAME);
+    }
+    if (database == null) {
+      throw new PSQLException(GT.tr("Database cannot be null"), PSQLState.INVALID_NAME);
+    }
 
     PGStream newStream = new PGStream(socketFactory, hostSpec, connectTimeout);
-
-    // Set the socket timeout if the "socketTimeout" property has been set.
-    int socketTimeout = PGProperty.SOCKET_TIMEOUT.getInt(info);
-    if (socketTimeout > 0) {
-      newStream.setNetworkTimeout(socketTimeout * 1000);
-    }
-
-    String maxResultBuffer = PGProperty.MAX_RESULT_BUFFER.get(info);
-    newStream.setMaxResultBuffer(maxResultBuffer);
-
-    // Enable TCP keep-alive probe if required.
-    boolean requireTCPKeepAlive = PGProperty.TCP_KEEP_ALIVE.getBoolean(info);
-    newStream.getSocket().setKeepAlive(requireTCPKeepAlive);
-
-    // Try to set SO_SNDBUF and SO_RECVBUF socket options, if requested.
-    // If receiveBufferSize and send_buffer_size are set to a value greater
-    // than 0, adjust. -1 means use the system default, 0 is ignored since not
-    // supported.
-
-    // Set SO_RECVBUF read buffer size
-    int receiveBufferSize = PGProperty.RECEIVE_BUFFER_SIZE.getInt(info);
-    if (receiveBufferSize > -1) {
-      // value of 0 not a valid buffer size value
-      if (receiveBufferSize > 0) {
-        newStream.getSocket().setReceiveBufferSize(receiveBufferSize);
-      } else {
-        LOGGER.log(Level.WARNING, "Ignore invalid value for receiveBufferSize: {0}", receiveBufferSize);
+    try {
+      // Set the socket timeout if the "socketTimeout" property has been set.
+      int socketTimeout = PGProperty.SOCKET_TIMEOUT.getInt(info);
+      if (socketTimeout > 0) {
+        newStream.setNetworkTimeout(socketTimeout * 1000);
       }
-    }
 
-    // Set SO_SNDBUF write buffer size
-    int sendBufferSize = PGProperty.SEND_BUFFER_SIZE.getInt(info);
-    if (sendBufferSize > -1) {
-      if (sendBufferSize > 0) {
-        newStream.getSocket().setSendBufferSize(sendBufferSize);
-      } else {
-        LOGGER.log(Level.WARNING, "Ignore invalid value for sendBufferSize: {0}", sendBufferSize);
+      String maxResultBuffer = PGProperty.MAX_RESULT_BUFFER.get(info);
+      newStream.setMaxResultBuffer(maxResultBuffer);
+
+      // Enable TCP keep-alive probe if required.
+      boolean requireTCPKeepAlive = PGProperty.TCP_KEEP_ALIVE.getBoolean(info);
+      newStream.getSocket().setKeepAlive(requireTCPKeepAlive);
+
+      // Enable TCP no delay if required
+      boolean requireTCPNoDelay = PGProperty.TCP_NO_DELAY.getBoolean(info);
+      newStream.getSocket().setTcpNoDelay(requireTCPNoDelay);
+
+      // Try to set SO_SNDBUF and SO_RECVBUF socket options, if requested.
+      // If receiveBufferSize and send_buffer_size are set to a value greater
+      // than 0, adjust. -1 means use the system default, 0 is ignored since not
+      // supported.
+
+      // Set SO_RECVBUF read buffer size
+      int receiveBufferSize = PGProperty.RECEIVE_BUFFER_SIZE.getInt(info);
+      if (receiveBufferSize > -1) {
+        // value of 0 not a valid buffer size value
+        if (receiveBufferSize > 0) {
+          newStream.getSocket().setReceiveBufferSize(receiveBufferSize);
+        } else {
+          LOGGER.log(Level.WARNING, "Ignore invalid value for receiveBufferSize: {0}",
+              receiveBufferSize);
+        }
       }
+
+      // Set SO_SNDBUF write buffer size
+      int sendBufferSize = PGProperty.SEND_BUFFER_SIZE.getInt(info);
+      if (sendBufferSize > -1) {
+        if (sendBufferSize > 0) {
+          newStream.getSocket().setSendBufferSize(sendBufferSize);
+        } else {
+          LOGGER.log(Level.WARNING, "Ignore invalid value for sendBufferSize: {0}", sendBufferSize);
+        }
+      }
+
+      if (LOGGER.isLoggable(Level.FINE)) {
+        LOGGER.log(Level.FINE, "Receive Buffer Size is {0}",
+            newStream.getSocket().getReceiveBufferSize());
+        LOGGER.log(Level.FINE, "Send Buffer Size is {0}",
+            newStream.getSocket().getSendBufferSize());
+      }
+
+      newStream = enableGSSEncrypted(newStream, gssEncMode, hostSpec.getHost(), info, connectTimeout);
+
+      // if we have a security context then gss negotiation succeeded. Do not attempt SSL
+      // negotiation
+      if (!newStream.isGssEncrypted()) {
+        // Construct and send an ssl startup packet if requested.
+        newStream = enableSSL(newStream, sslMode, info, connectTimeout);
+      }
+
+      // Make sure to set network timeout again, in case the stream changed due to GSS or SSL
+      if (socketTimeout > 0) {
+        newStream.setNetworkTimeout(socketTimeout * 1000);
+      }
+
+      List<String[]> paramList = getParametersForStartup(user, database, info);
+      sendStartupPacket(newStream, paramList);
+
+      // Do authentication (until AuthenticationOk).
+      doAuthentication(newStream, hostSpec.getHost(), user, info);
+
+      return newStream;
+    } catch (Exception e) {
+      closeStream(newStream);
+      throw e;
     }
-
-    if (LOGGER.isLoggable(Level.FINE)) {
-      LOGGER.log(Level.FINE, "Receive Buffer Size is {0}", newStream.getSocket().getReceiveBufferSize());
-      LOGGER.log(Level.FINE, "Send Buffer Size is {0}", newStream.getSocket().getSendBufferSize());
-    }
-
-    newStream = enableGSSEncrypted(newStream, gssEncMode, hostSpec.getHost(), user, info, connectTimeout);
-
-    // if we have a security context then gss negotiation succeeded. Do not attempt SSL negotiation
-    if (!newStream.isGssEncrypted()) {
-      // Construct and send an ssl startup packet if requested.
-      newStream = enableSSL(newStream, sslMode, info, connectTimeout);
-    }
-
-    // Make sure to set network timeout again, in case the stream changed due to GSS or SSL
-    if (socketTimeout > 0) {
-      newStream.setNetworkTimeout(socketTimeout * 1000);
-    }
-
-    List<String[]> paramList = getParametersForStartup(user, database, info);
-    sendStartupPacket(newStream, paramList);
-
-    // Do authentication (until AuthenticationOk).
-    doAuthentication(newStream, hostSpec.getHost(), user, info);
-
-    return newStream;
   }
 
   @Override
-  public QueryExecutor openConnectionImpl(HostSpec[] hostSpecs, String user, String database,
-      Properties info) throws SQLException {
+  public QueryExecutor openConnectionImpl(HostSpec[] hostSpecs, Properties info) throws SQLException {
     SslMode sslMode = SslMode.of(info);
     GSSEncMode gssEncMode = GSSEncMode.of(info);
 
@@ -210,7 +232,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
       PGStream newStream = null;
       try {
         try {
-          newStream = tryConnect(user, database, info, socketFactory, hostSpec, sslMode, gssEncMode);
+          newStream = tryConnect(info, socketFactory, hostSpec, sslMode, gssEncMode);
         } catch (SQLException e) {
           if (sslMode == SslMode.PREFER
               && PSQLState.INVALID_AUTHORIZATION_SPECIFICATION.getState().equals(e.getSQLState())) {
@@ -219,21 +241,18 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
             Throwable ex = null;
             try {
               newStream =
-                  tryConnect(user, database, info, socketFactory, hostSpec, SslMode.DISABLE,gssEncMode);
+                  tryConnect(info, socketFactory, hostSpec, SslMode.DISABLE,gssEncMode);
               LOGGER.log(Level.FINE, "Downgraded to non-encrypted connection for host {0}",
                   hostSpec);
-            } catch (SQLException ee) {
+            } catch (SQLException | IOException ee) {
               ex = ee;
-            } catch (IOException ee) {
-              ex = ee; // Can't use multi-catch in Java 6 :(
             }
+
             if (ex != null) {
               log(Level.FINE, "sslMode==PREFER, however non-SSL connection failed as well", ex);
               // non-SSL failed as well, so re-throw original exception
-              //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.1"
               // Add non-SSL exception as suppressed
               e.addSuppressed(ex);
-              //#endif
               throw e;
             }
           } else if (sslMode == SslMode.ALLOW
@@ -242,7 +261,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
             Throwable ex = null;
             try {
               newStream =
-                  tryConnect(user, database, info, socketFactory, hostSpec, SslMode.REQUIRE, gssEncMode);
+                  tryConnect(info, socketFactory, hostSpec, SslMode.REQUIRE, gssEncMode);
               LOGGER.log(Level.FINE, "Upgraded to encrypted connection for host {0}",
                   hostSpec);
             } catch (SQLException ee) {
@@ -253,10 +272,8 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
             if (ex != null) {
               log(Level.FINE, "sslMode==ALLOW, however SSL connection failed as well", ex);
               // non-SSL failed as well, so re-throw original exception
-              //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.1"
               // Add SSL exception as suppressed
               e.addSuppressed(ex);
-              //#endif
               throw e;
             }
 
@@ -270,8 +287,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
         // CheckerFramework can't infer newStream is non-nullable
         castNonNull(newStream);
         // Do final startup.
-        QueryExecutor queryExecutor = new QueryExecutorImpl(newStream, user, database,
-            cancelSignalTimeout, info);
+        QueryExecutor queryExecutor = new QueryExecutorImpl(newStream, cancelSignalTimeout, info);
 
         // Check Primary or Secondary
         HostStatus hostStatus = HostStatus.ConnectOK;
@@ -386,6 +402,9 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
   /**
    * Convert Java time zone to postgres time zone. All others stay the same except that GMT+nn
    * changes to GMT-nn and vise versa.
+   * If you provide GMT+/-nn postgres uses POSIX rules which has a positive sign for west of Greenwich
+   * JAVA uses ISO rules which the positive sign is east of Greenwich
+   * To make matters more interesting postgres will always report in ISO
    *
    * @return The current JVM time zone in postgresql format.
    */
@@ -411,7 +430,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
     return start + tz.substring(4);
   }
 
-  private PGStream enableGSSEncrypted(PGStream pgStream, GSSEncMode gssEncMode, String host, String user, Properties info,
+  private PGStream enableGSSEncrypted(PGStream pgStream, GSSEncMode gssEncMode, String host, Properties info,
                                     int connectTimeout)
       throws IOException, PSQLException {
 
@@ -424,10 +443,25 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
       return pgStream;
     }
 
-    String password = PGProperty.PASSWORD.get(info);
+    /*
+     at this point gssEncMode is either PREFER or REQUIRE
+     libpq looks to see if there is a ticket in the cache before asking
+     the server if it supports encrypted GSS connections or not.
+     since the user has specifically asked or either prefer or require we can
+     assume they want it.
+     */
+    /*
+    let's see if the server will allow a GSS encrypted connection
+     */
+    String user = PGProperty.USER.get(info);
+    if (user == null) {
+      throw new PSQLException("GSSAPI encryption required but was impossible user is null", PSQLState.CONNECTION_REJECTED);
+    }
+
+    // attempt to acquire a GSS encrypted connection
     LOGGER.log(Level.FINEST, " FE=> GSSENCRequest");
 
-    // Send SSL request packet
+    // Send GSSEncryption request packet
     pgStream.sendInteger4(8);
     pgStream.sendInteger2(1234);
     pgStream.sendInteger2(5680);
@@ -462,11 +496,14 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
       case 'G':
         LOGGER.log(Level.FINEST, " <=BE GSSEncryptedOk");
         try {
-          org.postgresql.gss.MakeGSS.authenticate(true, pgStream, host, user, password,
-              PGProperty.JAAS_APPLICATION_NAME.get(info),
-              PGProperty.KERBEROS_SERVER_NAME.get(info), false, // TODO: fix this
-              PGProperty.JAAS_LOGIN.getBoolean(info),
-              PGProperty.LOG_SERVER_ERROR_DETAIL.getBoolean(info));
+          AuthenticationPluginManager.withPassword(AuthenticationRequestType.GSS, info, password -> {
+            org.postgresql.gss.MakeGSS.authenticate(true, pgStream, host, user, password,
+                PGProperty.JAAS_APPLICATION_NAME.get(info),
+                PGProperty.KERBEROS_SERVER_NAME.get(info), false, // TODO: fix this
+                PGProperty.JAAS_LOGIN.getBoolean(info),
+                PGProperty.LOG_SERVER_ERROR_DETAIL.getBoolean(info));
+            return void.class;
+          });
           return pgStream;
         } catch (PSQLException ex) {
           // allow the connection to proceed
@@ -559,8 +596,8 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
     int length = 4 + 4;
     byte[][] encodedParams = new byte[params.size() * 2][];
     for (int i = 0; i < params.size(); ++i) {
-      encodedParams[i * 2] = params.get(i)[0].getBytes("UTF-8");
-      encodedParams[i * 2 + 1] = params.get(i)[1].getBytes("UTF-8");
+      encodedParams[i * 2] = params.get(i)[0].getBytes(StandardCharsets.UTF_8);
+      encodedParams[i * 2 + 1] = params.get(i)[1].getBytes(StandardCharsets.UTF_8);
       length += encodedParams[i * 2].length + 1 + encodedParams[i * 2 + 1].length + 1;
     }
 
@@ -583,15 +620,11 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
     // Now get the response from the backend, either an error message
     // or an authentication request
 
-    String password = PGProperty.PASSWORD.get(info);
-
     /* SSPI negotiation state, if used */
     ISSPIClient sspiClient = null;
 
-    //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.1"
     /* SCRAM authentication state, if used */
     org.postgresql.jre7.sasl.ScramAuthenticator scramAuthenticator = null;
-    //#endif
 
     try {
       authloop: while (true) {
@@ -628,23 +661,23 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
                   LOGGER.log(Level.FINEST, " <=BE AuthenticationReqMD5(salt={0})", Utils.toHexString(md5Salt));
                 }
 
-                if (password == null) {
-                  throw new PSQLException(
-                      GT.tr(
-                          "The server requested password-based authentication, but no password was provided."),
-                      PSQLState.CONNECTION_REJECTED);
-                }
-
-                byte[] digest =
-                    MD5Digest.encode(user.getBytes("UTF-8"), password.getBytes("UTF-8"), md5Salt);
+                byte[] digest = AuthenticationPluginManager.withEncodedPassword(
+                    AuthenticationRequestType.MD5_PASSWORD, info,
+                    encodedPassword -> MD5Digest.encode(user.getBytes(StandardCharsets.UTF_8),
+                        encodedPassword, md5Salt)
+                );
 
                 if (LOGGER.isLoggable(Level.FINEST)) {
-                  LOGGER.log(Level.FINEST, " FE=> Password(md5digest={0})", new String(digest, "US-ASCII"));
+                  LOGGER.log(Level.FINEST, " FE=> Password(md5digest={0})", new String(digest, StandardCharsets.US_ASCII));
                 }
 
-                pgStream.sendChar('p');
-                pgStream.sendInteger4(4 + digest.length + 1);
-                pgStream.send(digest);
+                try {
+                  pgStream.sendChar('p');
+                  pgStream.sendInteger4(4 + digest.length + 1);
+                  pgStream.send(digest);
+                } finally {
+                  java.util.Arrays.fill(digest, (byte) 0);
+                }
                 pgStream.sendChar(0);
                 pgStream.flush();
 
@@ -655,18 +688,12 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
                 LOGGER.log(Level.FINEST, "<=BE AuthenticationReqPassword");
                 LOGGER.log(Level.FINEST, " FE=> Password(password=<not shown>)");
 
-                if (password == null) {
-                  throw new PSQLException(
-                      GT.tr(
-                          "The server requested password-based authentication, but no password was provided."),
-                      PSQLState.CONNECTION_REJECTED);
-                }
-
-                byte[] encodedPassword = password.getBytes("UTF-8");
-
-                pgStream.sendChar('p');
-                pgStream.sendInteger4(4 + encodedPassword.length + 1);
-                pgStream.send(encodedPassword);
+                AuthenticationPluginManager.withEncodedPassword(AuthenticationRequestType.CLEARTEXT_PASSWORD, info, encodedPassword -> {
+                  pgStream.sendChar('p');
+                  pgStream.sendInteger4(4 + encodedPassword.length + 1);
+                  pgStream.send(encodedPassword);
+                  return void.class;
+                });
                 pgStream.sendChar(0);
                 pgStream.flush();
 
@@ -737,11 +764,14 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
                   castNonNull(sspiClient).startSSPI();
                 } else {
                   /* Use JGSS's GSSAPI for this request */
-                  org.postgresql.gss.MakeGSS.authenticate(false, pgStream, host, user, password,
-                      PGProperty.JAAS_APPLICATION_NAME.get(info),
-                      PGProperty.KERBEROS_SERVER_NAME.get(info), usespnego,
-                      PGProperty.JAAS_LOGIN.getBoolean(info),
-                      PGProperty.LOG_SERVER_ERROR_DETAIL.getBoolean(info));
+                  AuthenticationPluginManager.withPassword(AuthenticationRequestType.GSS, info, password -> {
+                    org.postgresql.gss.MakeGSS.authenticate(false, pgStream, host, user, password,
+                        PGProperty.JAAS_APPLICATION_NAME.get(info),
+                        PGProperty.KERBEROS_SERVER_NAME.get(info), usespnego,
+                        PGProperty.JAAS_LOGIN.getBoolean(info),
+                        PGProperty.LOG_SERVER_ERROR_DETAIL.getBoolean(info));
+                    return void.class;
+                  });
                 }
                 break;
 
@@ -755,25 +785,33 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
               case AUTH_REQ_SASL:
                 LOGGER.log(Level.FINEST, " <=BE AuthenticationSASL");
 
-                //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.1"
-                scramAuthenticator = new org.postgresql.jre7.sasl.ScramAuthenticator(user, castNonNull(password), pgStream);
+                scramAuthenticator = AuthenticationPluginManager.withPassword(AuthenticationRequestType.SASL, info, password -> {
+                  if (password == null) {
+                    throw new PSQLException(
+                        GT.tr(
+                            "The server requested SCRAM-based authentication, but no password was provided."),
+                        PSQLState.CONNECTION_REJECTED);
+                  }
+                  if (password.length == 0) {
+                    throw new PSQLException(
+                        GT.tr(
+                            "The server requested SCRAM-based authentication, but the password is an empty string."),
+                        PSQLState.CONNECTION_REJECTED);
+                  }
+                  return new org.postgresql.jre7.sasl.ScramAuthenticator(user, String.valueOf(password), pgStream);
+                });
                 scramAuthenticator.processServerMechanismsAndInit();
                 scramAuthenticator.sendScramClientFirstMessage();
                 // This works as follows:
                 // 1. When tests is run from IDE, it is assumed SCRAM library is on the classpath
                 // 2. In regular build for Java < 8 this `if` is deactivated and the code always throws
                 if (false) {
-                  //#else
                   throw new PSQLException(GT.tr(
                           "SCRAM authentication is not supported by this driver. You need JDK >= 8 and pgjdbc >= 42.2.0 (not \".jre\" versions)",
                           areq), PSQLState.CONNECTION_REJECTED);
-                  //#endif
-                  //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.1"
                 }
                 break;
-                //#endif
 
-              //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.1"
               case AUTH_REQ_SASL_CONTINUE:
                 castNonNull(scramAuthenticator).processServerFirstMessage(msgLen - 4 - 4);
                 break;
@@ -781,7 +819,6 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
               case AUTH_REQ_SASL_FINAL:
                 castNonNull(scramAuthenticator).verifyServerSignature(msgLen - 4 - 4);
                 break;
-              //#endif
 
               case AUTH_REQ_OK:
                 /* Cleanup after successful authentication */
@@ -826,6 +863,10 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
 
     final int dbVersion = queryExecutor.getServerVersionNum();
 
+    if (dbVersion >= ServerVersion.v9_0.getVersionNum()) {
+      SetupQueryRunner.run(queryExecutor, "BEGIN", false);
+    }
+
     if (dbVersion >= ServerVersion.v9_0.getVersionNum()) {
       SetupQueryRunner.run(queryExecutor, "SET extra_float_digits = 3", false);
     }
@@ -839,12 +880,49 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
       SetupQueryRunner.run(queryExecutor, sql.toString(), false);
     }
 
+    if (dbVersion >= ServerVersion.v9_0.getVersionNum()) {
+      SetupQueryRunner.run(queryExecutor, "COMMIT", false);
+    }
   }
 
+  /**
+   * Since PG14 there is GUC_REPORT ParamStatus {@code in_hot_standby} which is set to "on"
+   * when the server is in archive recovery or standby mode. In driver's lingo such server is called
+   * {@link org.postgresql.hostchooser.HostRequirement#secondary}.
+   * Previously {@code transaction_read_only} was used as a workable substitute.
+   * However {@code transaction_read_only} could have been manually overridden on the primary server
+   * by database user leading to a false positives: ie server is effectively read-only but
+   * technically is "primary" (not in a recovery/standby mode).
+   *
+   * <p>This method checks whether {@code in_hot_standby} GUC was reported by the server
+   * during initial connection:</p>
+   *
+   * <ul>
+   * <li>{@code in_hot_standby} was reported and the value was "on" then the server is a replica
+   * and database is read-only by definition, false is returned.</li>
+   * <li>{@code in_hot_standby} was reported and the value was "off"
+   * then the server is indeed primary but database may be in
+   * read-only mode nevertheless. We proceed to conservatively {@code show transaction_read_only}
+   * since users may not be expecting a readonly connection for {@code targetServerType=primary}</li>
+   * <li>If {@code in_hot_standby} has not been reported we fallback to pre v14 behavior.</li>
+   * </ul>
+   *
+   * <p>Do not confuse {@code hot_standby} and {@code in_hot_standby} ParamStatuses</p>
+   *
+   * @see <a href="https://www.postgresql.org/docs/current/protocol-flow.html#PROTOCOL-ASYNC">GUC_REPORT documentation</a>
+   * @see <a href="https://www.postgresql.org/docs/current/hot-standby.html">Hot standby documentation</a>
+   * @see <a href="https://www.postgresql.org/message-id/flat/1700970.cRWpxnom9y@hammer.magicstack.net">in_hot_standby patch thread v10</a>
+   * @see <a href="https://www.postgresql.org/message-id/flat/CAF3%2BxM%2B8-ztOkaV9gHiJ3wfgENTq97QcjXQt%2BrbFQ6F7oNzt9A%40mail.gmail.com">in_hot_standby patch thread v14</a>
+   *
+   */
   private boolean isPrimary(QueryExecutor queryExecutor) throws SQLException, IOException {
+    String inHotStandby = queryExecutor.getParameterStatus(IN_HOT_STANDBY);
+    if ("on".equalsIgnoreCase(inHotStandby)) {
+      return false;
+    }
     Tuple results = SetupQueryRunner.run(queryExecutor, "show transaction_read_only", true);
     Tuple nonNullResults = castNonNull(results);
-    String value = queryExecutor.getEncoding().decode(castNonNull(nonNullResults.get(0)));
-    return value.equalsIgnoreCase("off");
+    String queriedTransactionReadonly = queryExecutor.getEncoding().decode(castNonNull(nonNullResults.get(0)));
+    return queriedTransactionReadonly.equalsIgnoreCase("off");
   }
 }
