@@ -7,6 +7,7 @@ package org.postgresql.jdbc;
 
 import org.postgresql.core.BaseConnection;
 import org.postgresql.core.Oid;
+import org.postgresql.core.Parser;
 import org.postgresql.jdbc2.ArrayAssistant;
 import org.postgresql.jdbc2.ArrayAssistantRegistry;
 import org.postgresql.util.GT;
@@ -19,13 +20,14 @@ import org.postgresql.util.PSQLState;
 // import org.checkerframework.checker.nullness.qual.Nullable;
 
 import java.io.IOException;
-import java.io.UnsupportedEncodingException;
 import java.lang.reflect.Array;
 import java.math.BigDecimal;
 import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
+import java.nio.charset.StandardCharsets;
 import java.sql.SQLException;
 import java.sql.SQLFeatureNotSupportedException;
+import java.sql.Types;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -302,11 +304,7 @@ final class ArrayDecoding {
 
     @Override
     Object parseValue(String stringVal, BaseConnection connection) throws SQLException {
-      try {
-        return PGbytea.toBytes(stringVal.getBytes("ascii"));
-      } catch (UnsupportedEncodingException e) {
-        throw new java.lang.Error("ascii must be supported");
-      }
+      return PGbytea.toBytes(stringVal.getBytes(StandardCharsets.US_ASCII));
     }
   };
 
@@ -373,6 +371,8 @@ final class ArrayDecoding {
     OID_TO_DECODER.put(Oid.FLOAT4, FLOAT_OBJ_ARRAY);
     OID_TO_DECODER.put(Oid.TEXT, STRING_ARRAY);
     OID_TO_DECODER.put(Oid.VARCHAR, STRING_ARRAY);
+    // 42.2.x decodes jsonb array as String rather than PGobject
+    OID_TO_DECODER.put(Oid.JSONB, STRING_ONLY_DECODER);
     OID_TO_DECODER.put(Oid.BIT, BOOLEAN_OBJ_ARRAY);
     OID_TO_DECODER.put(Oid.BOOL, BOOLEAN_OBJ_ARRAY);
     OID_TO_DECODER.put(Oid.BYTEA, BYTE_ARRAY_ARRAY);
@@ -470,6 +470,11 @@ final class ArrayDecoding {
       throw org.postgresql.Driver.notImplemented(PgArray.class, "readArray(data,oid)");
     }
 
+    // 42.2.x should return enums as strings
+    int type = connection.getTypeInfo().getSQLType(typeName);
+    if (type == Types.CHAR || type == Types.VARCHAR) {
+      return (ArrayDecoder<A>) STRING_ONLY_DECODER;
+    }
     return (ArrayDecoder<A>) new MappedTypeObjectArrayDecoder(typeName);
   }
 
@@ -654,7 +659,7 @@ final class ArrayDecoding {
         insideString = !insideString;
         wasInsideString = true;
         continue;
-      } else if (!insideString && Character.isWhitespace(chars[i])) {
+      } else if (!insideString && Parser.isArrayWhiteSpace(chars[i])) {
         // white space
         continue;
       } else if ((!insideString && (chars[i] == delim || chars[i] == '}')) || i == chars.length - 1) {
