@@ -48,9 +48,9 @@ public enum GSSEncMode {
 
   public static GSSEncMode of(Properties info) throws PSQLException {
     String gssEncMode = PGProperty.GSS_ENC_MODE.get(info);
-    // If gssEncMode is not set, fallback to prefer
+    // If gssEncMode is not set, fallback to allow
     if (gssEncMode == null) {
-      return PREFER;
+      return ALLOW;
     }
 
     for (GSSEncMode mode : VALUES) {
