@@ -65,6 +65,10 @@ import java.sql.SQLXML;
 import java.sql.Time;
 import java.sql.Timestamp;
 import java.sql.Types;
+import java.time.LocalDate;
+import java.time.LocalDateTime;
+import java.time.LocalTime;
+import java.time.OffsetDateTime;
 import java.util.ArrayList;
 import java.util.Calendar;
 import java.util.Map;
@@ -132,14 +136,12 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
     return getUpdateCount();
   }
 
-  //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.2"
   @Override
   public long executeLargeUpdate() throws SQLException {
     executeWithFlags(QueryExecutor.QUERY_NO_RESULTS);
     checkNoResultUpdate();
     return getLargeUpdateCount();
   }
-  //#endif
 
   @Override
   public boolean execute(String sql) throws SQLException {
@@ -233,10 +235,8 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
         oid = Oid.DATE;
         break;
       case Types.TIME:
-      //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.2"
       case Types.TIME_WITH_TIMEZONE:
       case Types.TIMESTAMP_WITH_TIMEZONE:
-      //#endif
       case Types.TIMESTAMP:
         oid = Oid.UNSPECIFIED;
         break;
@@ -253,6 +253,9 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
       case Types.CLOB:
         oid = Oid.OID;
         break;
+      case Types.REF_CURSOR:
+        oid = Oid.REF_CURSOR;
+        break;
       case Types.ARRAY:
       case Types.DISTINCT:
       case Types.STRUCT:
@@ -334,6 +337,11 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
 
   public void setBigDecimal(/* @Positive */ int parameterIndex, /* @Nullable */ BigDecimal x)
       throws SQLException {
+    if (x != null && connection.binaryTransferSend(Oid.NUMERIC)) {
+      final byte[] bytes = ByteConverter.numeric(x);
+      bindBytes(parameterIndex, bytes, Oid.NUMERIC);
+      return;
+    }
     setNumber(parameterIndex, x);
   }
 
@@ -485,7 +493,12 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
 
     if ((x instanceof PGBinaryObject) && connection.binaryTransferSend(oid)) {
       PGBinaryObject binObj = (PGBinaryObject) x;
-      byte[] data = new byte[binObj.lengthInBytes()];
+      int length = binObj.lengthInBytes();
+      if (length == 0) {
+        preparedParameters.setNull(parameterIndex, oid);
+        return;
+      }
+      byte[] data = new byte[length];
       binObj.toBytes(data, 0);
       bindBytes(parameterIndex, data, oid);
     } else {
@@ -582,13 +595,11 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
           java.sql.Date tmpd;
           if (in instanceof java.util.Date) {
             tmpd = new java.sql.Date(((java.util.Date) in).getTime());
-            //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.2"
-          } else if (in instanceof java.time.LocalDate) {
-            setDate(parameterIndex, (java.time.LocalDate) in);
+          } else if (in instanceof LocalDate) {
+            setDate(parameterIndex, (LocalDate) in);
             break;
-            //#endif
           } else {
-            tmpd = connection.getTimestampUtils().toDate(getDefaultCalendar(), in.toString());
+            tmpd = getTimestampUtils().toDate(getDefaultCalendar(), in.toString());
           }
           setDate(parameterIndex, tmpd);
         }
@@ -600,13 +611,11 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
           java.sql.Time tmpt;
           if (in instanceof java.util.Date) {
             tmpt = new java.sql.Time(((java.util.Date) in).getTime());
-            //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.2"
-          } else if (in instanceof java.time.LocalTime) {
-            setTime(parameterIndex, (java.time.LocalTime) in);
+          } else if (in instanceof LocalTime) {
+            setTime(parameterIndex, (LocalTime) in);
             break;
-            //#endif
           } else {
-            tmpt = connection.getTimestampUtils().toTime(getDefaultCalendar(), in.toString());
+            tmpt = getTimestampUtils().toTime(getDefaultCalendar(), in.toString());
           }
           setTime(parameterIndex, tmpt);
         }
@@ -620,21 +629,18 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
           java.sql.Timestamp tmpts;
           if (in instanceof java.util.Date) {
             tmpts = new java.sql.Timestamp(((java.util.Date) in).getTime());
-            //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.2"
-          } else if (in instanceof java.time.LocalDateTime) {
-            setTimestamp(parameterIndex, (java.time.LocalDateTime) in);
+          } else if (in instanceof LocalDateTime) {
+            setTimestamp(parameterIndex, (LocalDateTime) in);
             break;
-            //#endif
           } else {
-            tmpts = connection.getTimestampUtils().toTimestamp(getDefaultCalendar(), in.toString());
+            tmpts = getTimestampUtils().toTimestamp(getDefaultCalendar(), in.toString());
           }
           setTimestamp(parameterIndex, tmpts);
         }
         break;
-      //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.2"
       case Types.TIMESTAMP_WITH_TIMEZONE:
-        if (in instanceof java.time.OffsetDateTime) {
-          setTimestamp(parameterIndex, (java.time.OffsetDateTime) in);
+        if (in instanceof OffsetDateTime) {
+          setTimestamp(parameterIndex, (OffsetDateTime) in);
         } else if (in instanceof PGTimestamp) {
           setObject(parameterIndex, in);
         } else {
@@ -644,7 +650,6 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
               PSQLState.INVALID_PARAMETER_TYPE);
         }
         break;
-      //#endif
       case Types.BOOLEAN:
       case Types.BIT:
         setBoolean(parameterIndex, BooleanTypeUtil.castToBoolean(in));
@@ -979,16 +984,14 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
       setPGobject(parameterIndex, (PGobject) x);
     } else if (x instanceof Character) {
       setString(parameterIndex, ((Character) x).toString());
-      //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.2"
-    } else if (x instanceof java.time.LocalDate) {
-      setDate(parameterIndex, (java.time.LocalDate) x);
-    } else if (x instanceof java.time.LocalTime) {
-      setTime(parameterIndex, (java.time.LocalTime) x);
-    } else if (x instanceof java.time.LocalDateTime) {
-      setTimestamp(parameterIndex, (java.time.LocalDateTime) x);
-    } else if (x instanceof java.time.OffsetDateTime) {
-      setTimestamp(parameterIndex, (java.time.OffsetDateTime) x);
-      //#endif
+    } else if (x instanceof LocalDate) {
+      setDate(parameterIndex, (LocalDate) x);
+    } else if (x instanceof LocalTime) {
+      setTime(parameterIndex, (LocalTime) x);
+    } else if (x instanceof LocalDateTime) {
+      setTimestamp(parameterIndex, (LocalDateTime) x);
+    } else if (x instanceof OffsetDateTime) {
+      setTimestamp(parameterIndex, (OffsetDateTime) x);
     } else if (x instanceof Map) {
       setMap(parameterIndex, (Map<?, ?>) x);
     } else if (x instanceof Number) {
@@ -1311,7 +1314,7 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
     if (connection.binaryTransferSend(Oid.DATE)) {
       byte[] val = new byte[4];
       TimeZone tz = cal != null ? cal.getTimeZone() : null;
-      connection.getTimestampUtils().toBinDate(tz, val, d);
+      getTimestampUtils().toBinDate(tz, val, d);
       preparedParameters.setBinaryParameter(i, val, Oid.DATE);
       return;
     }
@@ -1338,7 +1341,7 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
     if (cal == null) {
       cal = getDefaultCalendar();
     }
-    bindString(i, connection.getTimestampUtils().toString(cal, d), Oid.UNSPECIFIED);
+    bindString(i, getTimestampUtils().toString(cal, d), Oid.UNSPECIFIED);
   }
 
   public void setTime(/* @Positive */ int i, /* @Nullable */ Time t,
@@ -1366,7 +1369,7 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
     if (cal == null) {
       cal = getDefaultCalendar();
     }
-    bindString(i, connection.getTimestampUtils().toString(cal, t), oid);
+    bindString(i, getTimestampUtils().toString(cal, t), oid);
   }
 
   public void setTimestamp(/* @Positive */ int i, /* @Nullable */ Timestamp t,
@@ -1423,39 +1426,36 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
     if (cal == null) {
       cal = getDefaultCalendar();
     }
-    bindString(i, connection.getTimestampUtils().toString(cal, t), oid);
+    bindString(i, getTimestampUtils().toString(cal, t), oid);
   }
 
-  //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.2"
-  private void setDate(/* @Positive */ int i, java.time.LocalDate localDate) throws SQLException {
+  private void setDate(/* @Positive */ int i, LocalDate localDate) throws SQLException {
     int oid = Oid.DATE;
-    bindString(i, connection.getTimestampUtils().toString(localDate), oid);
+    bindString(i, getTimestampUtils().toString(localDate), oid);
   }
 
-  private void setTime(/* @Positive */ int i, java.time.LocalTime localTime) throws SQLException {
+  private void setTime(/* @Positive */ int i, LocalTime localTime) throws SQLException {
     int oid = Oid.TIME;
-    bindString(i, connection.getTimestampUtils().toString(localTime), oid);
+    bindString(i, getTimestampUtils().toString(localTime), oid);
   }
 
-  private void setTimestamp(/* @Positive */ int i, java.time.LocalDateTime localDateTime)
+  private void setTimestamp(/* @Positive */ int i, LocalDateTime localDateTime)
       throws SQLException {
     int oid = Oid.TIMESTAMP;
-    bindString(i, connection.getTimestampUtils().toString(localDateTime), oid);
+    bindString(i, getTimestampUtils().toString(localDateTime), oid);
   }
 
-  private void setTimestamp(/* @Positive */ int i, java.time.OffsetDateTime offsetDateTime)
+  private void setTimestamp(/* @Positive */ int i, OffsetDateTime offsetDateTime)
       throws SQLException {
     int oid = Oid.TIMESTAMPTZ;
-    bindString(i, connection.getTimestampUtils().toString(offsetDateTime), oid);
+    bindString(i, getTimestampUtils().toString(offsetDateTime), oid);
   }
-  //#endif
 
   public ParameterMetaData createParameterMetaData(BaseConnection conn, int[] oids)
       throws SQLException {
     return new PgParameterMetaData(conn, oids);
   }
 
-  //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.2"
   public void setObject(/* @Positive */ int parameterIndex, /* @Nullable */ Object x,
       java.sql.SQLType targetSqlType,
       int scaleOrLength) throws SQLException {
@@ -1467,7 +1467,6 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
       throws SQLException {
     throw Driver.notImplemented(this.getClass(), "setObject");
   }
-  //#endif
 
   public void setRowId(/* @Positive */ int parameterIndex, /* @Nullable */ RowId x) throws SQLException {
     throw Driver.notImplemented(this.getClass(), "setRowId(int, RowId)");
@@ -1643,11 +1642,10 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
   }
 
   private Calendar getDefaultCalendar() {
-    TimestampUtils timestampUtils = connection.getTimestampUtils();
-    if (timestampUtils.hasFastDefaultTimeZone()) {
-      return timestampUtils.getSharedCalendar(null);
+    if (getTimestampUtils().hasFastDefaultTimeZone()) {
+      return getTimestampUtils().getSharedCalendar(null);
     }
-    Calendar sharedCalendar = timestampUtils.getSharedCalendar(defaultTimeZone);
+    Calendar sharedCalendar = getTimestampUtils().getSharedCalendar(defaultTimeZone);
     if (defaultTimeZone == null) {
       defaultTimeZone = sharedCalendar.getTimeZone();
     }
