@@ -13,6 +13,7 @@ import org.postgresql.core.BaseStatement;
 import org.postgresql.core.CachedQuery;
 import org.postgresql.core.Field;
 import org.postgresql.core.ParameterList;
+import org.postgresql.core.Provider;
 import org.postgresql.core.Query;
 import org.postgresql.core.QueryExecutor;
 import org.postgresql.core.ResultCursor;
@@ -35,6 +36,7 @@ import java.sql.SQLWarning;
 import java.sql.Statement;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.TimeZone;
 import java.util.TimerTask;
 import java.util.concurrent.atomic.AtomicIntegerFieldUpdater;
 import java.util.concurrent.atomic.AtomicReferenceFieldUpdater;
@@ -145,6 +147,10 @@ public class PgStatement implements Statement, BaseStatement {
 
   protected int maxFieldSize = 0;
 
+  protected boolean adaptiveFetch = false;
+
+  private /* @Nullable */ TimestampUtils timestampUtils; // our own Object because it's not thread safe
+
   @SuppressWarnings("method.invocation.invalid")
   PgStatement(PgConnection c, int rsType, int rsConcurrency, int rsHoldability)
       throws SQLException {
@@ -154,6 +160,7 @@ public class PgStatement implements Statement, BaseStatement {
     concurrency = rsConcurrency;
     setFetchSize(c.getDefaultFetchSize());
     setPrepareThreshold(c.getPrepareThreshold());
+    setAdaptiveFetch(c.getAdaptiveFetch());
     this.rsHoldability = rsHoldability;
   }
 
@@ -162,7 +169,7 @@ public class PgStatement implements Statement, BaseStatement {
       /* @Nullable */ ResultCursor cursor) throws SQLException {
     PgResultSet newResult = new PgResultSet(originalQuery, this, fields, tuples, cursor,
         getMaxRows(), getMaxFieldSize(), getResultSetType(), getResultSetConcurrency(),
-        getResultSetHoldability());
+        getResultSetHoldability(), getAdaptiveFetch());
     newResult.setFetchSize(getFetchSize());
     newResult.setFetchDirection(getFetchDirection());
     return newResult;
@@ -332,7 +339,11 @@ public class PgStatement implements Statement, BaseStatement {
         PSQLState.WRONG_OBJECT_TYPE);
   }
 
-  private void closeUnclosedResults() throws SQLException {
+  /*
+  If there are multiple result sets we close any that have been processed and left open
+  by the client.
+   */
+  private void closeUnclosedProcessedResults() throws SQLException {
     synchronized (this) {
       ResultWrapper resultWrapper = this.firstUnclosedResult;
       ResultWrapper currentResult = this.result;
@@ -354,7 +365,11 @@ public class PgStatement implements Statement, BaseStatement {
 
     // Close any existing resultsets associated with this statement.
     synchronized (this) {
-      closeUnclosedResults();
+      closeUnclosedProcessedResults();
+
+      if ( this.result != null && this.result.getResultSet() != null ) {
+        this.result.getResultSet().close();
+      }
       result = null;
 
       ResultWrapper generatedKeys = this.generatedKeys;
@@ -425,11 +440,13 @@ public class PgStatement implements Statement, BaseStatement {
       }
     }
 
+    // Only use named statements after we hit the threshold. Note that only
+    // named statements can be transferred in binary format.
+    // isOneShotQuery will check to see if we have hith the prepareThreshold count
+
     if (isOneShotQuery(cachedQuery)) {
       flags |= QueryExecutor.QUERY_ONESHOT;
     }
-    // Only use named statements after we hit the threshold. Note that only
-    // named statements can be transferred in binary format.
 
     if (connection.getAutoCommit()) {
       flags |= QueryExecutor.QUERY_SUPPRESS_BEGIN;
@@ -471,7 +488,7 @@ public class PgStatement implements Statement, BaseStatement {
     try {
       startTimer();
       connection.getQueryExecutor().execute(queryToExecute, queryParameters, handler, maxrows,
-          fetchSize, flags);
+          fetchSize, flags, adaptiveFetch);
     } finally {
       killTimerTask();
     }
@@ -525,7 +542,7 @@ public class PgStatement implements Statement, BaseStatement {
     checkClosed();
     if (max < 0) {
       throw new PSQLException(
-          GT.tr("Maximum number of rows must be a value grater than or equal to 0."),
+          GT.tr("Maximum number of rows must be a value greater than or equal to 0."),
           PSQLState.INVALID_PARAMETER_VALUE);
     }
     maxrows = max;
@@ -868,7 +885,7 @@ public class PgStatement implements Statement, BaseStatement {
     try {
       startTimer();
       connection.getQueryExecutor().execute(queries, parameterLists, handler, maxrows, fetchSize,
-          flags);
+          flags, adaptiveFetch);
     } finally {
       killTimerTask();
       // There might be some rows generated even in case of failures
@@ -1039,7 +1056,6 @@ public class PgStatement implements Statement, BaseStatement {
     return forceBinaryTransfers;
   }
 
-  //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.2"
   @Override
   public long getLargeUpdateCount() throws SQLException {
     synchronized (this) {
@@ -1110,7 +1126,6 @@ public class PgStatement implements Statement, BaseStatement {
     }
     return getLargeUpdateCount();
   }
-  //#endif
 
   public boolean isClosed() throws SQLException {
     return isClosed;
@@ -1188,7 +1203,7 @@ public class PgStatement implements Statement, BaseStatement {
       // CLOSE_ALL_RESULTS
       if (current == Statement.CLOSE_ALL_RESULTS) {
         // Close preceding resultsets.
-        closeUnclosedResults();
+        closeUnclosedProcessedResults();
       }
 
       // Done.
@@ -1273,4 +1288,20 @@ public class PgStatement implements Statement, BaseStatement {
   protected void transformQueriesAndParameters() throws SQLException {
   }
 
+  @Override
+  public void setAdaptiveFetch(boolean adaptiveFetch) {
+    this.adaptiveFetch = adaptiveFetch;
+  }
+
+  @Override
+  public boolean getAdaptiveFetch() {
+    return adaptiveFetch;
+  }
+
+  protected TimestampUtils getTimestampUtils() {
+    if (timestampUtils == null) {
+      timestampUtils = new TimestampUtils(! connection.getQueryExecutor().getIntegerDateTimes(), (Provider<TimeZone>) new QueryExecutorTimeZoneProvider(connection.getQueryExecutor()));
+    }
+    return timestampUtils;
+  }
 }
