@@ -28,6 +28,8 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
+import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ConcurrentMap;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
@@ -98,9 +100,7 @@ public class TypeInfoCache implements TypeInfo {
       {"timestamp", Oid.TIMESTAMP, Types.TIMESTAMP, "java.sql.Timestamp", Oid.TIMESTAMP_ARRAY},
       {"timestamptz", Oid.TIMESTAMPTZ, Types.TIMESTAMP, "java.sql.Timestamp",
           Oid.TIMESTAMPTZ_ARRAY},
-      //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.2"
       {"refcursor", Oid.REF_CURSOR, Types.REF_CURSOR, "java.sql.ResultSet", Oid.REF_CURSOR_ARRAY},
-      //#endif
       {"json", Oid.JSON, Types.OTHER, "org.postgresql.util.PGobject", Oid.JSON_ARRAY},
       {"point", Oid.POINT, Types.OTHER, "org.postgresql.geometric.PGpoint", Oid.POINT_ARRAY}
   };
@@ -108,18 +108,29 @@ public class TypeInfoCache implements TypeInfo {
   /**
    * PG maps several alias to real type names. When we do queries against pg_catalog, we must use
    * the real type, not an alias, so use this mapping.
+   * <p>
+   * Additional values used at runtime (including case variants) will be added to the map.
+   * </p>
    */
-  private static final HashMap<String, String> typeAliases;
+  private static final ConcurrentMap<String, String> TYPE_ALIASES = new ConcurrentHashMap<>(20);
 
   static {
-    typeAliases = new HashMap<String, String>();
-    typeAliases.put("smallint", "int2");
-    typeAliases.put("integer", "int4");
-    typeAliases.put("int", "int4");
-    typeAliases.put("bigint", "int8");
-    typeAliases.put("float", "float8");
-    typeAliases.put("boolean", "bool");
-    typeAliases.put("decimal", "numeric");
+    TYPE_ALIASES.put("bool", "bool");
+    TYPE_ALIASES.put("boolean", "bool");
+    TYPE_ALIASES.put("smallint", "int2");
+    TYPE_ALIASES.put("int2", "int2");
+    TYPE_ALIASES.put("int", "int4");
+    TYPE_ALIASES.put("integer", "int4");
+    TYPE_ALIASES.put("int4", "int4");
+    TYPE_ALIASES.put("long", "int8");
+    TYPE_ALIASES.put("int8", "int8");
+    TYPE_ALIASES.put("bigint", "int8");
+    TYPE_ALIASES.put("float", "float4");
+    TYPE_ALIASES.put("float4", "float4");
+    TYPE_ALIASES.put("double", "float8");
+    TYPE_ALIASES.put("float8", "float8");
+    TYPE_ALIASES.put("decimal", "numeric");
+    TYPE_ALIASES.put("numeric", "numeric");
   }
 
   @SuppressWarnings("method.invocation.invalid")
@@ -268,7 +279,7 @@ public class TypeInfoCache implements TypeInfo {
         pgNameToSQLType.put(typeName, type);
       }
 
-      Integer typeOid = castNonNull(rs.getInt("oid"));
+      Integer typeOid = longOidToInt(castNonNull(rs.getLong("oid")));
       if (!oidToSQLType.containsKey(typeOid)) {
         oidToSQLType.put(typeOid, type);
       }
@@ -286,7 +297,25 @@ public class TypeInfoCache implements TypeInfo {
   }
 
   public synchronized int getSQLType(String pgTypeName) throws SQLException {
-    return getSQLType(castNonNull(getPGType(pgTypeName)));
+    /*
+    Get a few things out of the way such as arrays and known types
+     */
+    if (pgTypeName.endsWith("[]")) {
+      return Types.ARRAY;
+    }
+    Integer i = this.pgNameToSQLType.get(pgTypeName);
+    if (i != null) {
+      return i;
+    }
+
+    /*
+      All else fails then we will query the database.
+      save for future calls
+    */
+    i = getSQLType(castNonNull(getPGType(pgTypeName)));
+
+    pgNameToSQLType.put(pgTypeName, i);
+    return i;
   }
 
   public synchronized int getSQLType(int typeOid) throws SQLException {
@@ -299,11 +328,11 @@ public class TypeInfoCache implements TypeInfo {
       return i;
     }
 
-    LOGGER.log(Level.FINEST, "querying SQL typecode for pg type oid '{0}'", typeOid);
+    LOGGER.log(Level.FINEST, "querying SQL typecode for pg type oid '{0}'", intOidToLong(typeOid));
 
     PreparedStatement getTypeInfoStatement = prepareGetTypeInfoStatement();
 
-    getTypeInfoStatement.setInt(1, typeOid);
+    getTypeInfoStatement.setLong(1, intOidToLong(typeOid));
 
     // Go through BaseStatement to avoid transaction start.
     if (!((BaseStatement) getTypeInfoStatement)
@@ -430,6 +459,11 @@ public class TypeInfoCache implements TypeInfo {
   }
 
   public synchronized int getPGType(String pgTypeName) throws SQLException {
+    // there really isn't anything else to return other than UNSPECIFIED here.
+    if ( pgTypeName == null ) {
+      return Oid.UNSPECIFIED;
+    }
+
     Integer oid = pgNameToOid.get(pgTypeName);
     if (oid != null) {
       return oid;
@@ -515,7 +549,7 @@ public class TypeInfoCache implements TypeInfo {
     return getNameStatement;
   }
 
-  public int getPGArrayType(String elementTypeName) throws SQLException {
+  public int getPGArrayType(/* @Nullable */ String elementTypeName) throws SQLException {
     elementTypeName = getTypeForAlias(elementTypeName);
     return getPGType(elementTypeName + "[]");
   }
@@ -667,18 +701,21 @@ public class TypeInfoCache implements TypeInfo {
     return result == null ? "java.lang.String" : result;
   }
 
-  public String getTypeForAlias(String alias) {
-    String type = typeAliases.get(alias);
+  public /* @Nullable */ String getTypeForAlias(/* @Nullable */ String alias) {
+    if ( alias == null ) {
+      return null;
+    }
+    String type = TYPE_ALIASES.get(alias);
     if (type != null) {
       return type;
     }
-    if (alias.indexOf('"') == -1) {
-      type = typeAliases.get(alias.toLowerCase());
-      if (type != null) {
-        return type;
-      }
+    type = TYPE_ALIASES.get(alias.toLowerCase());
+    if (type == null) {
+      type = alias;
     }
-    return alias;
+    //populate for future use
+    TYPE_ALIASES.put(alias, type);
+    return type;
   }
 
   public int getPrecision(int oid, int typmod) {
@@ -962,4 +999,18 @@ public class TypeInfoCache implements TypeInfo {
     }
     return true;
   }
+
+  @Override
+  public int longOidToInt(long oid) throws SQLException {
+    if ((oid & 0xFFFF_FFFF_0000_0000L) != 0) {
+      throw new PSQLException(GT.tr("Value is not an OID: {0}", oid), PSQLState.NUMERIC_VALUE_OUT_OF_RANGE);
+    }
+
+    return (int) oid;
+  }
+
+  @Override
+  public long intOidToLong(int oid) {
+    return ((long) oid) & 0xFFFFFFFFL;
+  }
 }
