@@ -18,6 +18,7 @@ import org.postgresql.test.jdbc2.BaseTest4.BinaryMode;
 
 import org.junit.After;
 import org.junit.Assert;
+import org.junit.Assume;
 import org.junit.Before;
 import org.junit.Test;
 import org.junit.runner.RunWith;
@@ -87,6 +88,15 @@ public class DatabaseMetaDataTest {
     TestUtil.createCompositeType(con, "custom", "i int", false);
     TestUtil.createCompositeType(con, "_custom", "f float", false);
 
+    // create a table and multiple comments on it
+    TestUtil.createTable(con, "duplicate", "x text");
+    TestUtil.execute("comment on table duplicate is 'duplicate table'", con);
+    TestUtil.execute("create or replace function bar() returns integer language sql as $$ select 1 $$", con);
+    TestUtil.execute("comment on function bar() is 'bar function'", con);
+    try (Connection conPriv = TestUtil.openPrivilegedDB()) {
+      TestUtil.execute("update pg_description set objoid = 'duplicate'::regclass where objoid = 'bar'::regproc", conPriv);
+    }
+
     // 8.2 does not support arrays of composite types
     TestUtil.createTable(con, "customtable", "c1 custom, c2 _custom"
         + (TestUtil.haveMinimumServerVersion(con, ServerVersion.v8_3) ? ", c3 custom[], c4 _custom[]" : ""));
@@ -111,12 +121,22 @@ public class DatabaseMetaDataTest {
           "CREATE OR REPLACE FUNCTION f5() RETURNS TABLE (i int) LANGUAGE sql AS 'SELECT 1'");
     }
 
+    // create a custom `&` operator, which caused failure with `&` usage in getIndexInfo()
+    stmt.execute(
+        "CREATE OR REPLACE FUNCTION f6(numeric, integer) returns integer as 'BEGIN return $1::integer & $2;END;' language plpgsql immutable;");
+    stmt.execute("DROP OPERATOR IF EXISTS & (numeric, integer)");
+    stmt.execute("CREATE OPERATOR & (LEFTARG = numeric, RIGHTARG = integer, PROCEDURE = f6)");
+
     TestUtil.createDomain(con, "nndom", "int not null");
     TestUtil.createDomain(con, "varbit2", "varbit(3)");
     TestUtil.createDomain(con, "float83", "numeric(8,3)");
 
     TestUtil.createTable(con, "domaintable", "id nndom, v varbit2, f float83");
     stmt.close();
+
+    if ( TestUtil.haveMinimumServerVersion(con, ServerVersion.v12) ) {
+      TestUtil.createTable(con, "employee", "id int GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY, hours_per_week decimal(3,2), rate_per_hour decimal(3,2), gross_pay decimal GENERATED ALWAYS AS (hours_per_week * rate_per_hour) STORED");
+    }
   }
 
   @After
@@ -125,6 +145,8 @@ public class DatabaseMetaDataTest {
     // metadatatest table's type
     Statement stmt = con.createStatement();
     stmt.execute("DROP FUNCTION f4(int)");
+    TestUtil.execute("drop function bar()", con);
+    TestUtil.dropTable(con, "duplicate");
 
     TestUtil.dropView(con, "viewtest");
     TestUtil.dropTable(con, "metadatatest");
@@ -143,11 +165,17 @@ public class DatabaseMetaDataTest {
     stmt.execute("DROP FUNCTION f1(int, varchar)");
     stmt.execute("DROP FUNCTION f2(int, varchar)");
     stmt.execute("DROP FUNCTION f3(int, varchar)");
+    stmt.execute("DROP OPERATOR IF EXISTS & (numeric, integer)");
+    stmt.execute("DROP FUNCTION f6(numeric, integer)");
     TestUtil.dropTable(con, "domaintable");
     TestUtil.dropDomain(con, "nndom");
     TestUtil.dropDomain(con, "varbit2");
     TestUtil.dropDomain(con, "float83");
 
+    if ( TestUtil.haveMinimumServerVersion(con, ServerVersion.v12) ) {
+      TestUtil.dropTable(con, "employee");
+    }
+
     TestUtil.closeDB(con);
   }
 
@@ -612,11 +640,16 @@ public class DatabaseMetaDataTest {
     rs.close();
   }
 
-  @Test
-  public void testTablePrivileges() throws SQLException {
+  /*
+   * Helper function - this logic is used several times to test relation privileges
+   */
+  public void relationPrivilegesHelper(String relationName) throws SQLException {
+    // Query PG catalog for privileges
     DatabaseMetaData dbmd = con.getMetaData();
     assertNotNull(dbmd);
-    ResultSet rs = dbmd.getTablePrivileges(null, null, "metadatatest");
+    ResultSet rs = dbmd.getTablePrivileges(null, null, relationName);
+
+    // Parse result to check if table/view owner has select privileges
     boolean foundSelect = false;
     while (rs.next()) {
       if (rs.getString("GRANTEE").equals(TestUtil.getUser())
@@ -625,9 +658,32 @@ public class DatabaseMetaDataTest {
       }
     }
     rs.close();
-    // Test that the table owner has select priv
-    assertTrue("Couldn't find SELECT priv on table metadatatest for " + TestUtil.getUser(),
-        foundSelect);
+
+    // Check test condition
+    assertTrue("Couldn't find SELECT priv on relation "
+                + relationName + "  for " + TestUtil.getUser(),
+              foundSelect);
+  }
+
+  @Test
+  public void testTablePrivileges() throws SQLException {
+    relationPrivilegesHelper("metadatatest");
+  }
+
+  @Test
+  public void testViewPrivileges() throws SQLException {
+    relationPrivilegesHelper("viewtest");
+  }
+
+  @Test
+  public void testMaterializedViewPrivileges() throws SQLException {
+    Assume.assumeTrue(TestUtil.haveMinimumServerVersion(con, ServerVersion.v9_3));
+    TestUtil.createMaterializedView(con, "matviewtest", "SELECT id, quest FROM metadatatest");
+    try {
+      relationPrivilegesHelper("matviewtest");
+    } finally {
+      TestUtil.dropMaterializedView(con, "matviewtest");
+    }
   }
 
   @Test
@@ -640,24 +696,6 @@ public class DatabaseMetaDataTest {
     assertTrue(!rs.next());
   }
 
-  @Test
-  public void testViewPrivileges() throws SQLException {
-    DatabaseMetaData dbmd = con.getMetaData();
-    assertNotNull(dbmd);
-    ResultSet rs = dbmd.getTablePrivileges(null, null, "viewtest");
-    boolean foundSelect = false;
-    while (rs.next()) {
-      if (rs.getString("GRANTEE").equals(TestUtil.getUser())
-          && rs.getString("PRIVILEGE").equals("SELECT")) {
-        foundSelect = true;
-      }
-    }
-    rs.close();
-    // Test that the view owner has select priv
-    assertTrue("Couldn't find SELECT priv on table metadatatest for " + TestUtil.getUser(),
-        foundSelect);
-  }
-
   @Test
   public void testPrimaryKeys() throws SQLException {
     // At the moment just test that no exceptions are thrown KJ
@@ -748,6 +786,24 @@ public class DatabaseMetaDataTest {
     rs.close();
   }
 
+  @Test
+  public void testIndexInfoColumnCase() throws SQLException {
+    DatabaseMetaData dbmd = con.getMetaData();
+    assertNotNull(dbmd);
+
+    try (ResultSet rs = dbmd.getIndexInfo(null, null, "metadatatest", false, false)) {
+      ResultSetMetaData rsmd = rs.getMetaData();
+      for (int i = 1; i < rsmd.getColumnCount() + 1; i++) {
+        char[] chars = rsmd.getColumnName(i).toCharArray();
+        for (int j = 0; j < chars.length; j++) {
+          if (Character.isAlphabetic(chars[j])) {
+            assertTrue("Column: " + rsmd.getColumnName(i) + " is not UPPER CASE", Character.isUpperCase(chars[j]));
+          }
+        }
+      }
+    }
+  }
+
   @Test
   public void testNotNullDomainColumn() throws SQLException {
     DatabaseMetaData dbmd = con.getMetaData();
@@ -1341,9 +1397,32 @@ public class DatabaseMetaDataTest {
     assertTrue(!rs.next());
   }
 
+  @Test
+  public void testPartionedTablesIndex() throws SQLException {
+    if (TestUtil.haveMinimumServerVersion(con, ServerVersion.v11)) {
+      Statement stmt = null;
+      try {
+        stmt = con.createStatement();
+        stmt.execute(
+            "CREATE TABLE measurement (logdate date not null primary key,peaktemp int,unitsales int ) PARTITION BY RANGE (logdate);");
+        DatabaseMetaData dbmd = con.getMetaData();
+        ResultSet rs = dbmd.getPrimaryKeys("", "", "measurement");
+        assertTrue(rs.next());
+        assertEquals("measurement_pkey", rs.getString(6));
+
+      } finally {
+        if (stmt != null) {
+          stmt.execute("drop table if exists measurement");
+          stmt.close();
+        }
+      }
+    }
+
+  }
+
   @Test
   public void testPartitionedTables() throws SQLException {
-    if (TestUtil.haveMinimumServerVersion(con, ServerVersion.v10)) {
+    if (TestUtil.haveMinimumServerVersion(con, ServerVersion.v11)) {
       Statement stmt = null;
       try {
         stmt = con.createStatement();
@@ -1360,7 +1439,7 @@ public class DatabaseMetaDataTest {
 
       } finally {
         if (stmt != null) {
-          stmt.execute("drop table measurement");
+          stmt.execute("drop table if exists measurement");
           stmt.close();
         }
       }
@@ -1391,6 +1470,17 @@ public class DatabaseMetaDataTest {
     }
   }
 
+  @Test
+  public void testGeneratedColumns() throws SQLException {
+    if ( TestUtil.haveMinimumServerVersion(con, ServerVersion.v12) ) {
+      DatabaseMetaData dbmd = con.getMetaData();
+      ResultSet rs = dbmd.getColumns("", "", "employee", "gross_pay");
+      assertTrue(rs.next());
+      assertEquals(rs.getString("COLUMN_NAME"), "gross_pay");
+      assertTrue(rs.getBoolean("IS_GENERATEDCOLUMN"));
+    }
+  }
+
   @Test
   public void testGetSQLKeywords() throws SQLException {
     DatabaseMetaData dbmd = con.getMetaData();
