@@ -9,7 +9,6 @@ import org.postgresql.PGProperty;
 import org.postgresql.jdbc.GSSEncMode;
 import org.postgresql.jdbc.SslMode;
 import org.postgresql.test.TestUtil;
-import org.postgresql.test.jdbc2.BaseTest4;
 import org.postgresql.util.PSQLState;
 
 // import org.checkerframework.checker.nullness.qual.Nullable;
@@ -18,22 +17,21 @@ import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
 
-import java.io.File;
+import java.io.EOFException;
 import java.io.FileNotFoundException;
 import java.net.SocketException;
 import java.security.cert.CertPathValidatorException;
-import java.sql.ResultSet;
+import java.sql.Connection;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Collection;
-import java.util.Collections;
 import java.util.List;
 import java.util.Properties;
 
 import javax.net.ssl.SSLHandshakeException;
 
 @RunWith(Parameterized.class)
-public class SslTest extends BaseTest4 {
+public class SslTest {
   enum Hostname {
     GOOD("localhost"),
     BAD("127.0.0.1"),
@@ -113,25 +111,14 @@ public class SslTest extends BaseTest4 {
   public ClientRootCertificate clientRootCertificate;
 
   @Parameterized.Parameter(5)
-  public String certdir;
-
-  @Parameterized.Parameter(6)
   public GSSEncMode gssEncMode;
 
-  @Parameterized.Parameters(name = "host={0}, db={1} sslMode={2}, cCert={3}, cRootCert={4}, gssEncMode={6}")
+  @Parameterized.Parameters(name = "host={0}, db={1} sslMode={2}, cCert={3}, cRootCert={4}, gssEncMode={5}")
   public static Iterable<Object[]> data() {
-    Properties prop = TestUtil.loadPropertyFiles("ssltest.properties");
-    String enableSslTests = prop.getProperty("enable_ssl_tests");
-    if (!Boolean.parseBoolean(enableSslTests)) {
-      System.out.println("enableSslTests is " + enableSslTests + ", skipping SSL tests");
-      return Collections.emptyList();
-    }
+    TestUtil.assumeSslTestsEnabled();
 
     Collection<Object[]> tests = new ArrayList<Object[]>();
 
-    File certDirFile = TestUtil.getFile(prop.getProperty("certdir"));
-    String certdir = certDirFile.getAbsolutePath();
-
     for (SslMode sslMode : SslMode.VALUES) {
       for (Hostname hostname : Hostname.values()) {
         for (TestDatabase database : TestDatabase.VALUES) {
@@ -153,9 +140,11 @@ public class SslTest extends BaseTest4 {
                 continue;
               }
               for (GSSEncMode gssEncMode : GSSEncMode.values()) {
-                tests.add(
-                    new Object[]{hostname, database, sslMode, clientCertificate, rootCertificate,
-                        certdir, gssEncMode});
+                if (gssEncMode == GSSEncMode.REQUIRE) {
+                  // TODO: support gss tests in /certdir/pg_hba.conf
+                  continue;
+                }
+                tests.add(new Object[]{hostname, database, sslMode, clientCertificate, rootCertificate, gssEncMode});
               }
             }
           }
@@ -170,51 +159,6 @@ public class SslTest extends BaseTest4 {
     return value != null && value.contains(substring);
   }
 
-  @Override
-  protected void updateProperties(Properties props) {
-    super.updateProperties(props);
-    props.put(TestUtil.SERVER_HOST_PORT_PROP, host.value + ":" + TestUtil.getPort());
-    props.put(TestUtil.DATABASE_PROP, db.toString());
-    PGProperty.SSL_MODE.set(props, sslmode.value);
-    PGProperty.GSS_ENC_MODE.set(props, gssEncMode.value);
-    if (clientCertificate == ClientCertificate.EMPTY) {
-      PGProperty.SSL_CERT.set(props, "");
-      PGProperty.SSL_KEY.set(props, "");
-    } else {
-      PGProperty.SSL_CERT.set(props,
-          certdir + "/" + clientCertificate.fileName + ".crt");
-      PGProperty.SSL_KEY.set(props,
-          certdir + "/" + clientCertificate.fileName + ".pk8");
-    }
-    if (clientRootCertificate == ClientRootCertificate.EMPTY) {
-      PGProperty.SSL_ROOT_CERT.set(props, "");
-    } else {
-      PGProperty.SSL_ROOT_CERT.set(props,
-          certdir + "/" + clientRootCertificate.fileName + ".crt");
-    }
-  }
-
-  @Override
-  public void setUp() throws Exception {
-    SQLException e = null;
-    try {
-      super.setUp();
-    } catch (SQLException ex) {
-      e = ex;
-    }
-
-    try {
-      // Note that checkErrorCodes throws AssertionError for unexpected cases
-      checkErrorCodes(e);
-    } catch (AssertionError ae) {
-      // Make sure original SQLException is printed as well even in case of AssertionError
-      if (e != null) {
-        ae.initCause(e);
-      }
-      throw ae;
-    }
-  }
-
   private void assertClientCertRequired(SQLException e, String caseName) {
     if (e == null) {
       Assert.fail(caseName + " should result in failure of client validation");
@@ -308,11 +252,18 @@ public class SslTest extends BaseTest4 {
           Assert.fail(caseName + " ==> connection should be upgraded to SSL with no failures");
         }
       } else {
-        if (e == null) {
-          Assert.fail(caseName + " ==> connection should fail");
+        try {
+          if (e == null) {
+            Assert.fail(caseName + " ==> connection should fail");
+          }
+          Assert.assertEquals(caseName + " ==> INVALID_AUTHORIZATION_SPECIFICATION is expected",
+              PSQLState.INVALID_AUTHORIZATION_SPECIFICATION.getState(), e.getSQLState());
+        } catch (AssertionError er) {
+          for (AssertionError error : errors) {
+            er.addSuppressed(error);
+          }
+          throw er;
         }
-        Assert.assertEquals(caseName + " ==> INVALID_AUTHORIZATION_SPECIFICATION is expected",
-            PSQLState.INVALID_AUTHORIZATION_SPECIFICATION.getState(), e.getSQLState());
       }
       // ALLOW is ok
       return;
@@ -335,12 +286,7 @@ public class SslTest extends BaseTest4 {
 
     for (int i = 1; i < errors.size(); i++) {
       AssertionError error = errors.get(i);
-      // addSuppressed is Java 1.7+
-      //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.1"
       firstError.addSuppressed(error);
-      //#endif
-      // TODO: this is needed for Java < 1.7. Should it be just removed?
-      // error.printStackTrace();
     }
 
     throw firstError;
@@ -379,11 +325,8 @@ public class SslTest extends BaseTest4 {
       Assert.fail(caseName + " ==> exception should be caused by CertPathValidatorException,"
           + " but no CertPathValidatorException is present in the getCause chain");
     }
-    // getReason is Java 1.7+
-    //#if mvn.project.property.postgresql.jdbc.spec >= "JDBC4.1"
     Assert.assertEquals(caseName + " ==> CertPathValidatorException.getReason",
         "NO_TRUST_ANCHOR", validatorEx.getReason().toString());
-    //#endif
     return true;
   }
 
@@ -437,29 +380,47 @@ public class SslTest extends BaseTest4 {
     if (e == null) {
       Assert.fail(caseName + " should result in failure of client validation");
     }
-    Assert.assertEquals(caseName + " ==> CONNECTION_FAILURE is expected",
-        PSQLState.CONNECTION_FAILURE.getState(), e.getSQLState());
+    // Note: Java's SSLSocket handshake does NOT process alert messages
+    // even if they are present on the wire. This looks like a perfectly valid
+    // handshake, however, the subsequent read from the stream (e.g. during startup
+    // message) discovers the alert message (e.g. "Received fatal alert: decrypt_error")
+    // and converts that to exception.
+    // That is why "CONNECTION_UNABLE_TO_CONNECT" is listed here for BAD client cert.
+    // Ideally, hanshake failure should be detected during the handshake, not after sending the startup
+    // message
+    if (!PSQLState.CONNECTION_FAILURE.getState().equals(e.getSQLState())
+        && !(clientCertificate == ClientCertificate.BAD
+        && PSQLState.CONNECTION_UNABLE_TO_CONNECT.getState().equals(e.getSQLState()))
+    ) {
+      Assert.fail(caseName + " ==> CONNECTION_FAILURE(08006)"
+              + " or CONNECTION_UNABLE_TO_CONNECT(08001) is expected"
+              + ", got " + e.getSQLState());
+    }
 
-    // Two exceptions are possible
+    // Three exceptions are possible
     // SSLHandshakeException: Received fatal alert: unknown_ca
+    // EOFException
     // SocketException: broken pipe (write failed)
 
     // decrypt_error does not look to be a valid case, however, we allow it for now
     // SSLHandshakeException: Received fatal alert: decrypt_error
 
     SocketException brokenPipe = findCause(e, SocketException.class);
+    if (brokenPipe != null) {
+      if (!contains(brokenPipe.getMessage(), "Broken pipe")) {
+        Assert.fail(
+            caseName + " ==> server should have terminated the connection (broken pipe expected)"
+                + ", actual exception was " + brokenPipe.getMessage());
+      }
+      return true;
+    }
+
+    EOFException eofException = findCause(e, EOFException.class);
+    if (eofException != null) {
+      return true;
+    }
+
     SSLHandshakeException handshakeException = findCause(e, SSLHandshakeException.class);
-
-    if (brokenPipe == null && handshakeException == null) {
-      Assert.fail(caseName + " ==> exception should be caused by SocketException(broken pipe)"
-          + " or SSLHandshakeException. No exceptions of such kind are present in the getCause chain");
-    }
-    if (brokenPipe != null && !contains(brokenPipe.getMessage(), "Broken pipe")) {
-      Assert.fail(
-          caseName + " ==> server should have terminated the connection (broken pipe expected)"
-              + ", actual exception was " + brokenPipe.getMessage());
-    }
-
     if (handshakeException != null) {
       final String handshakeMessage = handshakeException.getMessage();
       if (!contains(handshakeMessage, "unknown_ca")
@@ -469,8 +430,13 @@ public class SslTest extends BaseTest4 {
                 + " ==> server should have terminated the connection (expected 'unknown_ca' or 'decrypt_error')"
                 + ", actual exception was " + handshakeMessage);
       }
+      return true;
     }
-    return true;
+
+    Assert.fail(caseName + " ==> exception should be caused by SocketException(broken pipe)"
+        + " or EOFException,"
+        + " or SSLHandshakeException. No exceptions of such kind are present in the getCause chain");
+    return false;
   }
 
   private static </* @Nullable */ T extends Throwable> T findCause(/* @Nullable */ Throwable t,
@@ -486,23 +452,40 @@ public class SslTest extends BaseTest4 {
 
   @Test
   public void run() throws SQLException {
-    if (con == null) {
-      // e.g. expected failure to connect
-      return;
-    }
-    ResultSet rs = con.createStatement().executeQuery("select ssl_is_used()");
-    Assert.assertTrue("select ssl_is_used() should return a row", rs.next());
-    boolean sslUsed = rs.getBoolean(1);
-    if (sslmode == SslMode.ALLOW) {
-      Assert.assertEquals("ssl_is_used: ",
-          db.requiresSsl(),
-          sslUsed);
+    Properties props = new Properties();
+    props.put(TestUtil.SERVER_HOST_PORT_PROP, host.value + ":" + TestUtil.getPort());
+    props.put(TestUtil.DATABASE_PROP, db.toString());
+    PGProperty.SSL_MODE.set(props, sslmode.value);
+    PGProperty.GSS_ENC_MODE.set(props, gssEncMode.value);
+    if (clientCertificate == ClientCertificate.EMPTY) {
+      PGProperty.SSL_CERT.set(props, "");
+      PGProperty.SSL_KEY.set(props, "");
     } else {
-      Assert.assertEquals("ssl_is_used: ",
-          sslmode != SslMode.DISABLE && !db.rejectsSsl(),
-          sslUsed);
+      PGProperty.SSL_CERT.set(props, TestUtil.getSslTestCertPath(clientCertificate.fileName + ".crt"));
+      PGProperty.SSL_KEY.set(props, TestUtil.getSslTestCertPath(clientCertificate.fileName + ".pk8"));
+    }
+    if (clientRootCertificate == ClientRootCertificate.EMPTY) {
+      PGProperty.SSL_ROOT_CERT.set(props, "");
+    } else {
+      PGProperty.SSL_ROOT_CERT.set(props, TestUtil.getSslTestCertPath(clientRootCertificate.fileName + ".crt"));
     }
-    TestUtil.closeQuietly(rs);
-  }
 
+    try (Connection conn = TestUtil.openDB(props)) {
+      boolean sslUsed = TestUtil.queryForBoolean(conn, "SELECT ssl_is_used()");
+      if (sslmode == SslMode.ALLOW) {
+        Assert.assertEquals("SSL should be used if the DB requires SSL", db.requiresSsl(), sslUsed);
+      } else {
+        Assert.assertEquals("SSL should be used unless it is disabled or the DB rejects it", sslmode != SslMode.DISABLE && !db.rejectsSsl(), sslUsed);
+      }
+    } catch (SQLException e) {
+      try {
+        // Note that checkErrorCodes throws AssertionError for unexpected cases
+        checkErrorCodes(e);
+      } catch (AssertionError ae) {
+        // Make sure original SQLException is printed as well even in case of AssertionError
+        ae.initCause(e);
+        throw ae;
+      }
+    }
+  }
 }
