@@ -7,12 +7,14 @@ package org.postgresql.util;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Queue;
+import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.logging.Handler;
 import java.util.logging.LogRecord;
 import java.util.regex.Pattern;
 
 public class TestLogHandler extends Handler {
-  public List<LogRecord> records = new ArrayList<LogRecord>();
+  public Queue<LogRecord> records = new ConcurrentLinkedQueue<LogRecord>();
 
   @Override
   public void publish(LogRecord record) {
@@ -28,7 +30,7 @@ public class TestLogHandler extends Handler {
   }
 
   public List<LogRecord> getRecordsMatching(Pattern messagePattern) {
-    ArrayList<LogRecord> matches = new ArrayList<LogRecord>();
+    List<LogRecord> matches = new ArrayList<LogRecord>();
     for (LogRecord r: this.records) {
       String message = r.getMessage();
       if (message != null && messagePattern.matcher(message).find()) {
